class ErrorCode:
    ELEMENT_CLICK_INTERCEPTED = [64, 'element click intercepted']
    ELEMENT_IS_NOT_SELECTABLE = [15, 'element not selectable']
    ELEMENT_NOT_INTERACTABLE = [60, 'element not interactable']
    ELEMENT_NOT_VISIBLE = [11, 'element not visible']
    IME_ENGINE_ACTIVATION_FAILED = [31, 'ime engine activation failed']
    IME_NOT_AVAILABLE = [30, 'ime not available']
    INSECURE_CERTIFICATE = ['insecure certificate']
    INVALID_ARGUMENT = [61, 'invalid argument']
    INVALID_COOKIE_DOMAIN = [24, 'invalid cookie domain']
    INVALID_COORDINATES = ['invalid coordinates']
    INVALID_ELEMENT_COORDINATES = [29, 'invalid element coordinates']
    INVALID_ELEMENT_STATE = [12, 'invalid element state']
    INVALID_SELECTOR = [32, 'invalid selector']
    INVALID_SESSION_ID = ['invalid session id']
    INVALID_XPATH_SELECTOR = [51, 'invalid selector']
    INVALID_XPATH_SELECTOR_RETURN_TYPER = [52, 'invalid selector']
    JAVASCRIPT_ERROR = [17, 'javascript error']
    METHOD_NOT_ALLOWED = [405, 'unsupported operation']
    MOVE_TARGET_OUT_OF_BOUNDS = [34, 'move target out of bounds']
    NO_ALERT_OPEN = [27, 'no such alert']
    NO_SUCH_COOKIE = [62, 'no such cookie']
    NO_SUCH_ELEMENT = [7, 'no such element']
    NO_SUCH_FRAME = [8, 'no such frame']
    NO_SUCH_SHADOW_ROOT = ['no such shadow root']
    NO_SUCH_WINDOW = [23, 'no such window']
    SCRIPT_TIMEOUT = [28, 'script timeout']
    SESSION_NOT_CREATED = [33, 'session not created']
    STALE_ELEMENT_REFERENCE = [10, 'stale element reference']
    SUCCESS = 0
    TIMEOUT = [21, 'timeout']
    UNABLE_TO_CAPTURE_SCREEN = [63, 'unable to capture screen']
    UNABLE_TO_SET_COOKIE = [25, 'unable to set cookie']
    UNEXPECTED_ALERT_OPEN = [26, 'unexpected alert open']
    UNKNOWN_COMMAND = [9, 'unknown command']
    UNKNOWN_ERROR = [13, 'unknown error']
    UNKNOWN_METHOD = ['unknown method exception']
    XPATH_LOOKUP_ERROR = [19, 'invalid selector']


class appiumby:
    ACCESSIBILITY_ID= 'accessibility id'
    ANDROID_DATA_MATCHER= '-android datamatcher'
    ANDROID_UIAUTOMATOR= '-android uiautomator'
    ANDROID_VIEWTAG= '-android viewtag'
    ANDROID_VIEW_MATCHER= '-android viewmatcher'
    CUSTOM= '-custom'
    IMAGE= '-image'
    IOS_CLASS_CHAIN= '-ios class chain'
    IOS_PREDICATE= '-ios predicate string'
    IOS_UIAUTOMATION= '-ios uiautomation'
    WINDOWS_UI_AUTOMATION= '-windows uiautomation'


class mobile_command:
    ACTIVATE_APP= 'activateApp'
    ACTIVATE_IME_ENGINE= 'activateIMEEngine'
    BACKGROUND= 'background'
    CLEAR= 'clear'
    CLOSE_APP= 'closeApp'
    COMPARE_IMAGES= 'compareImages'
    CONTEXTS= 'getContexts'
    DEACTIVATE_IME_ENGINE= 'deactivateIMEEngine'
    END_TEST_COVERAGE= 'endTestCoverage'
    EXECUTE_DRIVER= 'executeDriver'
    FINGER_PRINT= 'fingerPrint'
    GET_ACTIVE_IME_ENGINE= 'getActiveEngine'
    GET_ALL_SESSIONS= 'getAllSessions'
    GET_APP_STRINGS= 'getAppStrings'
    GET_AVAILABLE_IME_ENGINES= 'getAvailableIMEEngines'
    GET_AVAILABLE_LOG_TYPES= 'getAvailableLogTypes'
    GET_CLIPBOARD= 'getClipboard'
    GET_CURRENT_ACTIVITY= 'getCurrentActivity'
    GET_CURRENT_CONTEXT= 'getCurrentContext'
    GET_CURRENT_PACKAGE= 'getCurrentPackage'
    GET_DEVICE_TIME_GET= 'getDeviceTimeGet'
    GET_DEVICE_TIME_POST= 'getDeviceTimePost'
    GET_DISPLAY_DENSITY= 'getDisplayDensity'
    GET_EVENTS= 'getLogEvents'
    GET_LOCATION= 'getLocation'
    GET_LOG= 'getLog'
    GET_NETWORK_CONNECTION= 'getNetworkConnection'
    GET_PERFORMANCE_DATA= 'getPerformanceData'
    GET_PERFORMANCE_DATA_TYPES= 'getPerformanceDataTypes'
    GET_SESSION= 'getSession'
    GET_SETTINGS= 'getSettings'
    GET_SYSTEM_BARS= 'getSystemBars'
    HIDE_KEYBOARD= 'hideKeyboard'
    INSTALL_APP= 'installApp'
    IS_APP_INSTALLED= 'isAppInstalled'
    IS_IME_ACTIVE= 'isIMEActive'
    IS_KEYBOARD_SHOWN= 'isKeyboardShown'
    IS_LOCKED= 'isLocked'
    KEY_EVENT= 'keyEvent'
    LAUNCH_APP= 'launchApp'
    LOCATION_IN_VIEW= 'locationInView'
    LOCK= 'lock'
    LOG_EVENT= 'logCustomEvent'
    LONG_PRESS_KEYCODE= 'longPressKeyCode'
    MAKE_GSM_CALL= 'makeGsmCall'
    MULTI_ACTION= 'multiAction'
    OPEN_NOTIFICATIONS= 'openNotifications'
    PRESS_KEYCODE= 'pressKeyCode'
    PULL_FILE= 'pullFile'
    PULL_FOLDER= 'pullFolder'
    PUSH_FILE= 'pushFile'
    QUERY_APP_STATE= 'queryAppState'
    REMOVE_APP= 'removeApp'
    REPLACE_KEYS= 'replaceKeys'
    RESET= 'reset'
    SEND_SMS= 'sendSms'
    SET_CLIPBOARD= 'setClipboard'
    SET_GSM_SIGNAL= 'setGsmSignal'
    SET_GSM_VOICE= 'setGsmVoice'
    SET_IMMEDIATE_VALUE= 'setImmediateValue'
    SET_LOCATION= 'setLocation'
    SET_NETWORK_CONNECTION= 'setNetworkConnection'
    SET_NETWORK_SPEED= 'setNetworkSpeed'
    SET_POWER_AC= 'setPowerAc'
    SET_POWER_CAPACITY= 'setPowerCapacity'
    SHAKE= 'shake'
    START_ACTIVITY= 'startActivity'
    START_RECORDING_SCREEN= 'startRecordingScreen'
    STOP_RECORDING_SCREEN= 'stopRecordingScreen'
    SWITCH_TO_CONTEXT= 'switchToContext'
    TERMINATE_APP= 'terminateApp'
    TOGGLE_LOCATION_SERVICES= 'toggleLocationServices'
    TOGGLE_TOUCH_ID_ENROLLMENT= 'toggleTouchIdEnrollment'
    TOGGLE_WIFI= 'toggleWiFi'
    TOUCH_ACTION= 'touchAction'
    TOUCH_ID= 'touchId'
    UNLOCK= 'unlock'
    UPDATE_SETTINGS= 'updateSettings'


# Licensed to the Software Freedom Conservancy (SFC) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The SFC licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
from selenium.webdriver.common.actions import interaction
from selenium.webdriver.common.actions.input_device import InputDevice

from typing import Union

from selenium.webdriver.remote.webelement import WebElement


class ScrollOrigin:

    def __init__(self, origin: Union[str, WebElement], x_offset: int, y_offset: int) -> None:
        self._origin = origin
        self._x_offset = x_offset
        self._y_offset = y_offset

    @classmethod
    def from_element(cls, element: WebElement, x_offset: int = 0, y_offset: int = 0):
        return cls(element, x_offset, y_offset)

    @classmethod
    def from_viewport(cls, x_offset: int = 0, y_offset: int = 0):
        return cls('viewport', x_offset, y_offset)

    @property
    def origin(self) -> Union[str, WebElement]:
        return self._origin

    @property
    def x_offset(self) -> int:
        return self._x_offset

    @property
    def y_offset(self) -> int:
        return self._y_offset


class WheelInput(InputDevice):

    def __init__(self, name) -> None:
        super().__init__(name=name)
        self.name = name
        self.type = interaction.WHEEL

    def encode(self) -> dict:
        return {"type": self.type,
                "id": self.name,
                "actions": [acts for acts in self.actions]}

    def create_scroll(self, x: int, y: int, delta_x: int,
                      delta_y: int, duration: int, origin) -> None:
        if isinstance(origin, WebElement):
            origin = {"element-6066-11e4-a52e-4f735466cecf": origin.id}
        self.add_action({"type": "scroll", "x": x, "y": y, "deltaX": delta_x,
                         "deltaY": delta_y, "duration": duration,
                         "origin": origin})

    def create_pause(self, pause_duration: Union[int, float]) -> None:
        self.add_action(
            {"type": "pause", "duration": int(pause_duration * 1000)})



from select import select
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver import ActionChains
from selenium.webdriver.common.alert import Alert
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support.select import Select
import os

class customwebDriverwait:
    def __init__(self, driver):
        #( "this is correct")
        self.driver = driver
        self.__wait=10

    '''An expectation for checking that an element is present on the DOM of a page and visible. 
    Visibility means that the element is not only displayed but also has a height and width that is greater than 0.
     locator - used to find the element returns the WebElement once it is located and visible'''

    def WaitFor_VisibilityOf_Element_Located(self, xpath):
        try:
            ##print(self.customWait,'------>wait')
            return WebDriverWait(self.driver, self.customWait).until(EC.visibility_of_element_located((By.XPATH, xpath)))
        except Exception as error:
            raise error
    def WaitFor_invisibilityOf_Element_Located(self, xpath):
        try:
            ##print(self.customWait,'------>wait')
            return WebDriverWait(self.driver, self.customWait).until(EC.invisibility_of_element_located((By.XPATH, xpath)))
        except Exception as error:
            raise error            
    '''An expectation for checking that an element is present on the DOM of a page. 
    This does not necessarily mean that the element is visible. 
    locator - used to find the element returns the WebElement once it is located'''

    def WaitFor_PresenseOf_Element_Located(self, xpath):
        try:
            print(self.customWait,"___________custom")
            return WebDriverWait(self.driver, self.customWait).until(EC.presence_of_element_located((By.XPATH, xpath)))
        except Exception as error:
            raise error
    '''An Expectation for checking an element is visible and enabled such that you can click it.

    element is either a locator (text) or an WebElement'''

    def WaitForElement_tobe_Clickable(self, xpath):
        try:
            return WebDriverWait(self.driver, self.customWait).until(EC.element_to_be_clickable((By.XPATH, xpath)))
        except Exception as error:
            raise error    

    def WaitForAlert_is_present(self):
        try:
            return WebDriverWait(self.driver, self.customWait).until(EC.alert_is_present())
        except Exception as error:
            raise error
    '''An expectation for checking if the given attribute is included in the specified element. 
    locator, attribute'''

    def WaitFor_element_attribute_to_include(self, xpath, value):
        try:
            return WebDriverWait(self.driver, self.customWait).until(EC.element_attribute_to_include((By.XPATH, xpath), value))
        except Exception as error:
            raise error        
    '''An expectation to locate an element and check if the selection state specified is in that state. 
    locator is a tuple of (by, path) is_selected is a boolean'''

    def WaitFor_element_located_selection_state_to_be(self, xpath, is_Selected):
        try:
            return WebDriverWait(self.driver, self.customWait).until(EC.element_located_selection_state_to_be((By.XPATH, xpath), is_Selected))
        except Exception as error:
            raise error
    '''An expectation for the element to be located is selected. locator is a tuple of (by, path)'''

    def WaitFor_element_located_to_be_selected(self, xpath):
        try:
            return WebDriverWait(self.driver, self.customWait).until(EC.element_located_to_be_selected((By.XPATH, xpath)))
        except Exception as error:
            raise error
    '''An expectation for checking if the given element is selected. 
    element is WebElement object is_selected is a Boolean.”'''

    def WaitFor_element_selection_state_to_be(self, xpath, is_Selected):
        try:
            return WebDriverWait(self.driver, self.customWait).until(EC.element_selection_state_to_be((By.XPATH, xpath), is_Selected))
        except Exception as error:
            raise error
    '''An expectation for checking the selection is selected. 
    element is WebElement object'''

    def WaitFor_element_to_be_selected(self, xpath):
        try:
            if(customwebDriverwait.WaitFor_VisibilityOf_Element_Located(xpath)):
                target = customwebDriverwait.WaitFor_VisibilityOf_Element_Located(
                    xpath)
                return WebDriverWait(self.driver, self.customWait).until(EC.element_to_be_selected(target))
        except Exception as error:
            raise error
    '''An expectation for checking whether the given frame is available to switch to. 
    If the frame is available it switches the given driver to the specified frame.'''

    def WaitFor_frame_to_be_available_and_switch_to_it(self, xpath):
        try:
            return WebDriverWait(self.driver, self.customWait).until(EC.frame_to_be_available_and_switch_to_it((By.XPATH, xpath)))
        except Exception as error:
            raise error            
    '''
    yet to cover the following

    selenium.webdriver.support.expected_conditions.invisibility_of_element(element)
    An Expectation for checking that an element is either invisible or not present on the DOM.

    element is either a locator (text) or an WebElement

    selenium.webdriver.support.expected_conditions.invisibility_of_element_located(locator)
    An Expectation for checking that an element is either invisible or not present on the DOM.

    locator used to find the eleme


    '''
    '''An expectation that a new window will be opened and have the number of windows handles increase'''

    def WaitFor_new_window_is_opened(self):
        try:
            currentWindowHandles = self.driver.window_handles
            return WebDriverWait(self.driver, self.customWait).until(EC.new_window_is_opened(currentWindowHandles))
        except Exception as error:
            raise error
    '''An expectation for the number of windows to be a certain value.'''

    def WaitFor_number_of_windows_to_be(self, number):
        try:
        # currentWindowHandles=self.driver.window_handles
            return WebDriverWait(self.driver, self.customWait).until(EC.number_of_windows_to_be(number))
        except Exception as error:
            raise error
    '''An expectation for checking that there is at least one element present on a web page. 
    locator is used to find the element returns the list of WebElements once they are located'''

    def WaitFor_presence_of_all_elements_located(self, xpath):
        try:
        # currentWindowHandles=self.driver.window_handles
            return WebDriverWait(self.driver, self.customWait).until(EC.presence_of_all_elements_located((By.XPATH, xpath)))
        except Exception as error:
            raise error
    '''Wait until an element is no longer attached to the DOM. element is the element to wait for. 
    returns False if the element is still attached to the DOM, true otherwise.'''

    def WaitFor_staleness_of(self, xpath):
        try:
            element = self.driver.find_element_by_id(xpath)
            return WebDriverWait(self.driver, self.customWait).until(EC.staleness_of(element))
        except Exception as error:
            raise error
    '''An expectation for checking if the given text is present in the specified element. locator, text'''

    def WaitFor_text_to_be_present_in_element(self, xpath, text_):
        try:
            return WebDriverWait(self.driver, self.customWait).until(EC.text_to_be_present_in_element((By.XPATH, xpath), text_))
        except Exception as error:
            raise error
    '''An expectation for checking if the given text is present in the element’s attribute. 
    locator, attribute, text'''

    def WaitFor_text_to_be_present_in_element_attribute(self, xpath, attribute, text_):
        try:
            return WebDriverWait(self.driver, self.customWait).until(EC.text_to_be_present_in_element_attribute((By.XPATH, xpath), attribute, text_))
        except Exception as error:
            raise error
    '''An expectation for checking if the given text is present in the element’s value. locator, text'''

    def WaitFor_text_to_be_present_in_element_value(self, xpath, text_):
        try:
            return WebDriverWait(self.driver, self.customWait).until(EC.text_to_be_present_in_element_value((By.XPATH, xpath), text_))
        except Exception as error:
            raise error
    '''An expectation for checking that the title contains a case-sensitive substring. 
    title is the fragment of title expected returns True when the title matches, False otherwise'''

    def WaitFor_title_contains(self, text_):
        try:
        # page_title=self.driver.title
            return WebDriverWait(self.driver, self.customWait).until(EC.title_contains(text_))
        except Exception as error:
            raise error
    '''An expectation for checking the title of a page. title is the expected title, 
    which must be an exact match returns True if the title matches, false otherwise.'''

    def WaitFor_title_is(self, text_):
        try:
            # page_title=self.driver.title
            return WebDriverWait(self.driver, self.customWait).until(EC.title_is(text_))
        except Exception as error:
            raise error
    '''An expectation for checking the current url. url is the expected url, 
    which must not be an exact match returns True if the url is different, false otherwise.'''

    def WaitFor_url_changes(self, url):
        try:
            # page_title=self.driver.title
            return WebDriverWait(self.driver, self.customWait).until(EC.url_changes(url))
        except Exception as error:
            raise error
    '''An expectation for checking that the current url contains a case-sensitive substring. 
    url is the fragment of url expected, returns True when the url matches, False otherwise'''

    def WaitFor_url_contains(self, url):
        try:
            # page_title=self.driver.title
            return WebDriverWait(self.driver, self.customWait).until(EC.url_contains(url))
        except Exception as error:
            raise error
    '''An expectation for checking the current url. pattern is the expected pattern, 
    which must be an exact match returns True if the url matches, false otherwise.'''

    def WaitFor_url_matches(self, pattern):
        try:
            # page_title=self.driver.title
            return WebDriverWait(self.driver, self.customWait).until(EC.url_matches(pattern))
        except Exception as error:
            raise error
    '''An expectation for checking the current url. url is the expected url, 
    which must be an exact match returns True if the url matches, false otherwise.'''

    def WaitFor_url_to_be(self, url):
        try:
            # page_title=self.driver.title
            return WebDriverWait(self.driver, self.customWait).until(EC.url_to_be(url))
        except Exception as error:
            raise error
    '''An expectation for checking that an element, known to be present on the DOM of a page, is visible. 
    Visibility means that the element is not only displayed but also has a height and width that is greater than 0. 
    element is the WebElement returns the (same) WebElement once it is visible'''

    def WaitFor_visibility_of(self, xpath):
        try:
            element = self.driver.find_element_by_id(xpath)
            return WebDriverWait(self.driver, self.customWait).until(EC.visibility_of(element))
        except Exception as error:
            raise error
    '''An expectation for checking that all elements are present on the DOM of a page and visible. 
    Visibility means that the elements are not only displayed but also has a height and width that is greater than 0. 
    locator - used to find the elements returns the list of WebElements once they are located and visible'''

    def WaitFor_visibility_of_all_elements_located(self, xpath):
        try:
            return WebDriverWait(self.driver, self.customWait).until(EC.visibility_of_all_elements_located((By.XPATH, xpath)))
        except Exception as error:
            raise error
    '''An expectation for checking that there is at least one element visible on a web page. 
    locator is used to find the element returns the list of WebElements once they are located'''

    def WaitFor_visibility_of_any_elements_located(self, xpath):
        try:
            return WebDriverWait(self.driver, self.customWait).until(EC.visibility_of_any_elements_located((By.XPATH, xpath)))
        except Exception as error:
            raise error
    def driver_implicit_wait(self,wait):
        try:
            self.driver.implicitly_wait(wait)
        except Exception as error:
            raise error    
    @property
    def customWait(self):
         ##print("getter method called")
         return self.__wait
       
     # a setter function
    @customWait.setter
    def customWait(self, wait=None):
         ##print("custom wait is set to=",wait)
         if(wait!=None):
            self.__wait = wait
         else:
            wait=10
            self.__wait=wait
    def WaitFor_PresenseOf_Element_Located_AnyLocatorType(self, locatorType:str,locator):
        #self.locatorType=str(locatorType).upper()
        ##print(self.locatorType,"------>")
        try:
            if(locatorType.upper()=='XPATH'):
                #print("matching xpath",locatorType,'<---->')
                return WebDriverWait(self.driver, self.customWait).until(EC.presence_of_element_located((By.XPATH, locator)))
            elif(locatorType.upper()=='ID'):
                return WebDriverWait(self.driver, self.customWait).until(EC.presence_of_element_located((By.ID, locator)))
            elif(locatorType.upper()=='CLASS_NAME'):
                return WebDriverWait(self.driver, self.customWait).until(EC.presence_of_element_located((By.CLASS_NAME, locator)))
            elif(locatorType.upper()=='CSS_SELECTOR'):
                return WebDriverWait(self.driver, self.customWait).until(EC.presence_of_element_located((By.CSS_SELECTOR, locator)))
            elif(locatorType.upper()=='LINK_TEXT'):
                return WebDriverWait(self.driver, self.customWait).until(EC.presence_of_element_located((By.LINK_TEXT, locator))) 
            elif(locatorType.upper()=='NAME'):
                return WebDriverWait(self.driver, self.customWait).until(EC.presence_of_element_located((By.NAME, locator)))
            elif(locatorType.upper()=='PARTIAL_LINK_TEXT'):
                return WebDriverWait(self.driver, self.customWait).until(EC.presence_of_element_located((By.PARTIAL_LINK_TEXT, locator)))                                                                               
            elif(locatorType.upper()=='TAG_NAME'):
                return WebDriverWait(self.driver, self.customWait).until(EC.presence_of_element_located((By.TAG_NAME, locator)))
            else:
                raise Exception(" ** wrong selector/ no selector ** ",locatorType.upper())   
        except Exception as error:
            raise error 
    def WaitFor_Visibility_of_Element_Located_AnyLocatorType(self, locatorType:str,locator):
        #self.locatorType=str(locatorType).upper()
        ##print(self.locatorType,"------>")
        try:
            if(locatorType.upper()=='XPATH'):
                return WebDriverWait(self.driver, self.customWait).until(EC.visibility_of_element_located((By.XPATH, locator)))
            elif(locatorType.upper()=='ID'):
                return WebDriverWait(self.driver, self.customWait).until(EC.visibility_of_element_located((By.ID, locator)))
            elif(locatorType.upper()=='CLASS_NAME'):
                return WebDriverWait(self.driver, self.customWait).until(EC.visibility_of_element_located((By.CLASS_NAME, locator)))
            elif(locatorType.upper()=='CSS_SELECTOR'):
                return WebDriverWait(self.driver, self.customWait).until(EC.visibility_of_element_located((By.CSS_SELECTOR, locator)))
            elif(locatorType.upper()=='LINK_TEXT'):
                return WebDriverWait(self.driver, self.customWait).until(EC.visibility_of_element_located((By.LINK_TEXT, locator))) 
            elif(locatorType.upper()=='NAME'):
                return WebDriverWait(self.driver, self.customWait).until(EC.visibility_of_element_located((By.NAME, locator)))
            elif(locatorType.upper()=='PARTIAL_LINK_TEXT'):
                return WebDriverWait(self.driver, self.customWait).until(EC.visibility_of_element_located((By.PARTIAL_LINK_TEXT, locator)))                                                                               
            elif(locatorType.upper()=='TAG_NAME'):
                return WebDriverWait(self.driver, self.customWait).until(EC.visibility_of_element_located((By.TAG_NAME, locator)))
            else:
                raise Exception(" ** wrong selector/ no selector ** ",locatorType.upper())   
        except Exception as error:
            raise error
    def WaitFor_invisibility_of_Element_Located_AnyLocatorType(self, locatorType:str,locator):
        #self.locatorType=str(locatorType).upper()
        ##print(self.locatorType,"------>")
        try:
            if(locatorType.upper()=='XPATH'):
                return WebDriverWait(self.driver, self.customWait).until(EC.invisibility_of_element_located((By.XPATH, locator)))
            elif(locatorType.upper()=='ID'):
                return WebDriverWait(self.driver, self.customWait).until(EC.invisibility_of_element_located((By.ID, locator)))
            elif(locatorType.upper()=='CLASS_NAME'):
                return WebDriverWait(self.driver, self.customWait).until(EC.invisibility_of_element_located((By.CLASS_NAME, locator)))
            elif(locatorType.upper()=='CSS_SELECTOR'):
                return WebDriverWait(self.driver, self.customWait).until(EC.invisibility_of_element_located((By.CSS_SELECTOR, locator)))
            elif(locatorType.upper()=='LINK_TEXT'):
                return WebDriverWait(self.driver, self.customWait).until(EC.invisibility_of_element_located((By.LINK_TEXT, locator))) 
            elif(locatorType.upper()=='NAME'):
                return WebDriverWait(self.driver, self.customWait).until(EC.invisibility_of_element_located((By.NAME, locator)))
            elif(locatorType.upper()=='PARTIAL_LINK_TEXT'):
                return WebDriverWait(self.driver, self.customWait).until(EC.invisibility_of_element_located((By.PARTIAL_LINK_TEXT, locator)))                                                                               
            elif(locatorType.upper()=='TAG_NAME'):
                return WebDriverWait(self.driver, self.customWait).until(EC.invisibility_of_element_located((By.TAG_NAME, locator)))
            else:
                raise Exception(" ** wrong selector/ no selector ** ",locatorType.upper())   
        except Exception as error:
            raise error            
    def WaitFor_presence_of_all_elements_located_AnyLocatorType(self, locatorType:str,locator):                                    
        #self.locatorType=str(locatorType).upper()
        try:
            if(locatorType.upper()=='XPATH'):
                ##print("matching xpath--")
                return WebDriverWait(self.driver, self.customWait).until(EC.presence_of_all_elements_located((By.XPATH, locator)))
            elif(locatorType.upper()=='ID'):
                return WebDriverWait(self.driver, self.customWait).until(EC.presence_of_all_elements_located((By.ID, locator)))
            elif(locatorType.upper()=='CLASS_NAME'):
                return WebDriverWait(self.driver, self.customWait).until(EC.presence_of_all_elements_located((By.CLASS_NAME, locator)))
            elif(locatorType.upper()=='CSS_SELECTOR'):
                return WebDriverWait(self.driver, self.customWait).until(EC.presence_of_all_elements_located((By.CSS_SELECTOR, locator)))
            elif(locatorType.upper()=='LINK_TEXT'):
                return WebDriverWait(self.driver, self.customWait).until(EC.presence_of_all_elements_located((By.LINK_TEXT, locator))) 
            elif(locatorType.upper()=='NAME'):
                return WebDriverWait(self.driver, self.customWait).until(EC.presence_of_all_elements_located((By.NAME, locator)))
            elif(locatorType.upper()=='PARTIAL_LINK_TEXT'):
                return WebDriverWait(self.driver, self.customWait).until(EC.presence_of_all_elements_located((By.PARTIAL_LINK_TEXT, locator)))                                                                               
            elif(locatorType.upper()=='TAG_NAME'):
                return WebDriverWait(self.driver, self.customWait).until(EC.presence_of_all_elements_located((By.TAG_NAME, locator)))
            else:
                raise Exception(" ** wrong selector/ no selector ** ",locatorType.upper())
        except Exception as error:
            raise error
    def WaitFor_frame_to_be_available_and_switch_to_it_AnyLocator(self, locatorType:str,locator):
        #self.LocatorType=str(locatorType).capitalize
        try:
            if(locatorType.upper()=="XPATH"):
                return WebDriverWait(self.driver, self.customWait).until(EC.frame_to_be_available_and_switch_to_it((By.XPATH, locator)))
            elif(locatorType.upper()=="ID"):
                return WebDriverWait(self.driver, self.customWait).until(EC.frame_to_be_available_and_switch_to_it((By.ID, locator)))
            elif(locatorType.upper()=="CLASS_NAME"):
                return WebDriverWait(self.driver, self.customWait).until(EC.frame_to_be_available_and_switch_to_it((By.CLASS_NAME, locator)))
            elif(locatorType.upper()=="CSS_SELECTOR"):
                return WebDriverWait(self.driver, self.customWait).until(EC.frame_to_be_available_and_switch_to_it((By.CSS_SELECTOR, locator)))
            elif(locatorType.upper()=="LINK_TEXT"):
                return WebDriverWait(self.driver, self.customWait).until(EC.frame_to_be_available_and_switch_to_it((By.LINK_TEXT, locator)))
            elif(locatorType.upper()=="NAME"):
                return WebDriverWait(self.driver, self.customWait).until(EC.frame_to_be_available_and_switch_to_it((By.NAME, locator)))
            elif(locatorType.upper()=="PARTIAL_LINK_TEXT"):
                return WebDriverWait(self.driver, self.customWait).until(EC.frame_to_be_available_and_switch_to_it((By.PARTIAL_LINK_TEXT, locator)))                                                                               
            elif(locatorType.upper()=="TAG_NAME"):
                return WebDriverWait(self.driver, self.customWait).until(EC.frame_to_be_available_and_switch_to_it((By.TAG_NAME, locator)))
            else:
                raise Exception(" ** wrong selector/ no selector ** ",locatorType.upper())    
        except Exception as error:
            raise error        
              


from distutils.log import error
from pyparsing import And
from selenium.webdriver import ActionChains
from selenium.webdriver.common.alert import Alert
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support.select import Select
from pyallied.web.webWaits import customwebDriverwait
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from seletools.actions import drag_and_drop
import os




class DragAndDrop(customwebDriverwait):
    # drag and drop
    def __init__(self, driver):
        super().__init__(driver)
        self.driver = driver
        '''
        drag_and_drop(source, target)¶
        Holds down the left mouse button on the source element,
        then moves to the target element and releases the mouse button.
        Args:	
        source: The element to mouse down.
        target: The element to mouse up
        '''
    def dragAndDrop(self, sourcexpath, destinationxpath):
        try:
            SourceelElementPresense=super().WaitFor_PresenseOf_Element_Located(sourcexpath)
            SourceelElementVisibility=super().WaitFor_VisibilityOf_Element_Located(sourcexpath)
            DestinationElementPresense=super().WaitFor_PresenseOf_Element_Located(destinationxpath)
            DestinationElementVisibility=super().WaitFor_VisibilityOf_Element_Located(destinationxpath)
            if SourceelElementPresense :
                if  DestinationElementPresense :
                    if SourceelElementVisibility :
                        if DestinationElementVisibility:
                            sourceElement = self.driver.find_element(By.XPATH, sourcexpath)
                            destination = self.driver.find_element(By.XPATH, destinationxpath)
                            action_chains = ActionChains(self.driver)
                            action_chains.drag_and_drop(sourceElement, destination).perform()

                        else:
                            self.__raise_element_not_visible_exception(destinationxpath)
                    else:
                        self.__raise_element_not_visible_exception(sourcexpath) 
                else:
                    self.__raise_element_not_present_exception(destinationxpath)
            else:
                self.__raise_element_not_present_exception(sourcexpath)                           
        except Exception as error:
            raise error            
        '''
        drag_and_drop_by_offset(source, xoffset, yoffset)¶
        Holds down the left mouse button on the source element,
        then moves to the target offset and releases the mouse button.
        Args:	
        source: The element to mouse down.
        xoffset: X offset to move to.
        yoffset: Y offset to move to.
        '''
    def dragAndDropByOffset(self, sourcexpath, xoffset, yoffset):
        try:
            SourceelElementPresense=super().WaitFor_PresenseOf_Element_Located(sourcexpath)
            SourceelElementVisibility=super().WaitFor_VisibilityOf_Element_Located(sourcexpath)
            if(SourceelElementPresense):
                if(SourceelElementVisibility):
                    action_chains = ActionChains(self.driver)
                    action_chains.drag_and_drop_by_offset(
                        SourceelElementVisibility, xoffset, yoffset).pause(5).perform()
                else:
                    self.__raise_element_not_visible_exception(sourcexpath) 
            else:
                self.__raise_element_not_present_exception(sourcexpath)                        
        except Exception as error:
            raise error
    def dragElement_ToOffSet_And_Drop(self, sourcexpath, xoffset, yoffset):
        try:
            SourceelElementPresense=super().WaitFor_PresenseOf_Element_Located(sourcexpath)
            SourceelElementVisibility=super().WaitFor_VisibilityOf_Element_Located(sourcexpath)
            if(SourceelElementPresense):
                if(SourceelElementVisibility):
                    action_chains = ActionChains(self.driver)

                    action_chains.move_to_element(SourceelElementVisibility).pause(1).click_and_hold().pause(2).move_by_offset(xoffset, yoffset).pause(2).release().perform()
                else:
                    self.__raise_element_not_visible_exception(sourcexpath) 
            else:
                self.__raise_element_not_present_exception(sourcexpath)        
        except Exception as error:
            raise error
    def seletools_drag_and_drop_xpath(self, sourcexpath, destinationxpath):
        try:
            SourceelElementPresense=super().WaitFor_PresenseOf_Element_Located(sourcexpath)
            SourceelElementVisibility=super().WaitFor_VisibilityOf_Element_Located(sourcexpath)
            DestinationElementPresense=super().WaitFor_PresenseOf_Element_Located(destinationxpath)
            DestinationElementVisibility=super().WaitFor_VisibilityOf_Element_Located(destinationxpath)
            if(SourceelElementPresense  ):
                if DestinationElementPresense:
                    if SourceelElementVisibility:
                        if DestinationElementVisibility:                  
                            drag_and_drop(self.driver, SourceelElementVisibility, DestinationElementVisibility)
                        else:
                            self.__raise_element_not_visible_exception(destinationxpath)
                    else:
                        self.__raise_element_not_visible_exception(sourcexpath)
                else:
                    self.__raise_element_not_present_exception(destinationxpath)
            else:
                self.__raise_element_not_present_exception(sourcexpath)                


        except Exception as error:
            raise error
    def seletools_drag_and_drop_css(self, source_css, destination_css):
        try:
            SourceelElementPresense=self.__waitFor_presenseOf_element_located(source_css)
            SourceelElementVisibility=self.__waitFor_visibilityOf_element_located(source_css)
            DestinationElementPresense=self.__waitFor_presenseOf_element_located(destination_css)
            DestinationElementVisibility=self.__waitFor_visibilityOf_element_located(destination_css)
            if(SourceelElementPresense  ):
                if DestinationElementPresense:
                    if SourceelElementVisibility:
                        if DestinationElementVisibility:                   
                            drag_and_drop(self.driver, SourceelElementVisibility, DestinationElementVisibility) 
                        else:
                            self.__raise_element_not_visible_exception(destination_css)
                    else:
                        self.__raise_element_not_visible_exception(source_css)
                else:
                    self.__raise_element_not_present_exception(destination_css)
            else:
                self.__raise_element_not_present_exception(source_css)                                       
        except Exception as error:
            raise error            
    '''
    ATTENTION: you must use CSS selectors for draggable and droppable elements in order for this script to work. 
    It’s related to the way that browser search elements in DOM tree using embedded scripts
    '''        
    def dragAndDrop_cssSelectorOnly_jsExecuter(self,source_css, destination_css):
        try:
            #file_path=os.path.dirname(__file__)
            f = open(os.path.dirname(__file__)+"\dragDrop.js",  "r")
            javascript = f.read()
            f.close()
            self.driver.execute_script(javascript, source_css, destination_css)
        except Exception as error:
            raise error
    def moveToElement_WithOffset_and_drag_and_drop(self, xpath, xoffset_click, yoffset_click,xoffset_move,yoffset_move):
        try:
            actions = ActionChains(self.driver)
            elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
            elementVisible = super().WaitFor_VisibilityOf_Element_Located(xpath)
            if elementPresense:
                if elementVisible:
                    #element = self.driver.find_element(By.XPATH, xpath)
                    actions.move_to_element_with_offset(elementVisible, xoffset, yoffset).click_and_hold().move_by_offset(xoffset_move,yoffset_move).release().pause(1).perform()
                    #actions.perform()
                else:
                    self.__raise_element_not_visible_exception(xpath)
            else:
                self.__raise_element_not_present_exception(xpath)                     
        except Exception as error:
            raise error   

    def drag_and_drop_with_move_to_element(self, xpath_source,xpath_destination ):
        try:
            actions = ActionChains(self.driver)
            elementPresense_source = super().WaitFor_PresenseOf_Element_Located(xpath_source)
            elementVisible_source = super().WaitFor_VisibilityOf_Element_Located(xpath_source)
            elementPresense_destination = super().WaitFor_PresenseOf_Element_Located(xpath_destination)
            elementVisible_destination = super().WaitFor_VisibilityOf_Element_Located(xpath_destination)            
            if elementPresense_source:
                if elementVisible_source:
                    if elementPresense_destination:
                        if elementVisible_destination:

                    #element = self.driver.find_element(By.XPATH, xpath)
                    #actions.move_to_element_with_offset(elementVisible, xoffset, yoffset).click_and_hold().move_by_offset(xoffset_move,yoffset_move).release().pause(1).perform()
                                actions.move_to_element(elementVisible_source).click_and_hold().move_to_element(elementVisible_destination).click().release().pause(1).perform()
                    #actions.perform()
                        else:
                            self.__raise_element_not_visible_exception(xpath_destination)
                    else:
                        self.__raise_element_not_present_exception(xpath_destination)
                else:
                    self.__raise_element_not_visible_exception(xpath_source) 
            else:
                self.__raise_element_not_present_exception(xpath_source)                                      
        except Exception as error:
            raise error  

    def __waitFor_presenseOf_element_located(self, cssLocator):
        try:
            return WebDriverWait(self.driver, self.customWait).until(EC.presence_of_element_located((By.CSS_SELECTOR, cssLocator)))
        except Exception as error:
            raise error
    def __waitFor_visibilityOf_element_located(self, cssLocator):
        try:
            #print(self.customWait,'------>wait')
            return WebDriverWait(self.driver, self.customWait).until(EC.visibility_of_element_located((By.CSS_SELECTOR, cssLocator)))
        except Exception as error:
            raise error
    def __raise_element_not_present_exception(self,locator):
        raise Exception("element not present : "+locator)

    def __raise_element_not_visible_exception(self,locator):
        raise Exception("element not visible : "+locator) 


class key_codes:
    # ACTION_DOWN	getAction() value: the key has been pressed down.
    ACTION_DOWN: int = 0
    # ACTION_MULTIPLE	getAction() value: multiple duplicate key events have occurred in a row, or a complex string is being delivered.
    ACTION_MULTIPLE: int = 2
    # ACTION_UP	getAction() value: the key has been released.
    ACTION_UP: int = 1
    # FLAG_CANCELED	When associated with up key events, this indicates that the key press has been canceled.
    FLAG_CANCELED: int = 32
    # FLAG_CANCELED_LONG_PRESS	Set when a key event has FLAG_CANCELED set because a long press action was executed while it was down.
    FLAG_CANCELED_LONG_PRESS: int = 256
    # FLAG_EDITOR_ACTION	This mask is used for compatibility, to identify enter keys that are coming from an IME whose enter key has been auto-labelled "next" or "done".
    FLAG_EDITOR_ACTION: int = 16
    # FLAG_FALLBACK	Set when a key event has been synthesized to implement default behavior for an event that the application did not handle.
    FLAG_FALLBACK: int = 1024
    # FLAG_FROM_SYSTEM	This mask is set if an event was known to come from a trusted part of the system.
    FLAG_FROM_SYSTEM: int = 8
    # FLAG_KEEP_TOUCH_MODE	This mask is set if we don't want the key event to cause us to leave touch mode.
    FLAG_KEEP_TOUCH_MODE: int = 4
    # FLAG_LONG_PRESS	This flag is set for the first key repeat that occurs after the long press timeout.
    FLAG_LONG_PRESS: int = 128
    # FLAG_SOFT_KEYBOARD	This mask is set if the key event was generated by a software keyboard.
    FLAG_SOFT_KEYBOARD: int = 2
    # FLAG_TRACKING	Set for ACTION_UP when this event's key code is still being tracked from its initial down.
    FLAG_TRACKING: int = 512
    # FLAG_VIRTUAL_HARD_KEY	This key event was generated by a virtual (on-screen) hard key area.
    FLAG_VIRTUAL_HARD_KEY: int = 64
    # FLAG_WOKE_HERE	This mask is set if the device woke because of this key event.
    FLAG_WOKE_HERE: int = 1
    KEYCODE_0: int = 7  # KEYCODE_0	Key code constant: '0' key.
    KEYCODE_1: int = 8  # KEYCODE_1	Key code constant: '1' key.
    KEYCODE_2: int = 9  # KEYCODE_2	Key code constant: '2' key.
    KEYCODE_3: int = 10  # KEYCODE_3	Key code constant: '3' key.
    # KEYCODE_3D_MODE	Key code constant: 3D Mode key.
    KEYCODE_3D_MODE: int = 206
    KEYCODE_4: int = 11  # KEYCODE_4	Key code constant: '4' key.
    KEYCODE_5: int = 12  # KEYCODE_5	Key code constant: '5' key.
    KEYCODE_6: int = 13  # KEYCODE_6	Key code constant: '6' key.
    KEYCODE_7: int = 14  # KEYCODE_7	Key code constant: '7' key.
    KEYCODE_8: int = 15  # KEYCODE_8	Key code constant: '8' key.
    KEYCODE_9: int = 16  # KEYCODE_9	Key code constant: '9' key.
    KEYCODE_A: int = 29  # KEYCODE_A	Key code constant: 'A' key.
    # KEYCODE_ALT_LEFT	Key code constant: Left Alt modifier key.
    KEYCODE_ALT_LEFT: int = 57
    # KEYCODE_ALT_RIGHT	Key code constant: Right Alt modifier key.
    KEYCODE_ALT_RIGHT: int = 58
    KEYCODE_APOSTROPHE: int = 75  # KEYCODE_APOSTROPHE	Key code constant: ''' (apostrophe) key.
    KEYCODE_APP_SWITCH: int = 187  # KEYCODE_APP_SWITCH	Key code constant: App switch key.
    KEYCODE_ASSIST: int = 219  # KEYCODE_ASSIST	Key code constant: Assist key.
    KEYCODE_AT: int = 77  # KEYCODE_AT	Key code constant: '@' key.
    # KEYCODE_AVR_INPUT	Key code constant: A/V Receiver input key.
    KEYCODE_AVR_INPUT: int = 182
    # KEYCODE_AVR_POWER	Key code constant: A/V Receiver power key.
    KEYCODE_AVR_POWER: int = 181
    KEYCODE_B: int = 30  # KEYCODE_B	Key code constant: 'B' key.
    KEYCODE_BACK: int = 4  # KEYCODE_BACK	Key code constant: Back key.
    # KEYCODE_BACKSLASH	Key code constant: '\' key.
    KEYCODE_BACKSLASH: int = 73
    # KEYCODE_BOOKMARK	Key code constant: Bookmark key.
    KEYCODE_BOOKMARK: int = 174
    # KEYCODE_BREAK	Key code constant: Break / Pause key.
    KEYCODE_BREAK: int = 121
    # KEYCODE_BUTTON_1	Key code constant: Generic Game Pad Button 	1
    KEYCODE_BUTTON_1: int = 188
    # KEYCODE_BUTTON_10	Key code constant: Generic Game Pad Button 	10
    KEYCODE_BUTTON_10: int = 197
    # KEYCODE_BUTTON_11	Key code constant: Generic Game Pad Button 	11
    KEYCODE_BUTTON_11: int = 198
    # KEYCODE_BUTTON_12	Key code constant: Generic Game Pad Button 	12
    KEYCODE_BUTTON_12: int = 199
    # KEYCODE_BUTTON_13	Key code constant: Generic Game Pad Button 	13
    KEYCODE_BUTTON_13: int = 200
    # KEYCODE_BUTTON_14	Key code constant: Generic Game Pad Button 	14
    KEYCODE_BUTTON_14: int = 201
    # KEYCODE_BUTTON_15	Key code constant: Generic Game Pad Button 	15
    KEYCODE_BUTTON_15: int = 202
    # KEYCODE_BUTTON_16	Key code constant: Generic Game Pad Button 	16
    KEYCODE_BUTTON_16: int = 203
    # KEYCODE_BUTTON_2	Key code constant: Generic Game Pad Button 	2
    KEYCODE_BUTTON_2: int = 189
    # KEYCODE_BUTTON_3	Key code constant: Generic Game Pad Button 	3
    KEYCODE_BUTTON_3: int = 190
    # KEYCODE_BUTTON_4	Key code constant: Generic Game Pad Button 	4
    KEYCODE_BUTTON_4: int = 191
    # KEYCODE_BUTTON_5	Key code constant: Generic Game Pad Button 	5
    KEYCODE_BUTTON_5: int = 192
    # KEYCODE_BUTTON_6	Key code constant: Generic Game Pad Button 	6
    KEYCODE_BUTTON_6: int = 193
    # KEYCODE_BUTTON_7	Key code constant: Generic Game Pad Button 	7
    KEYCODE_BUTTON_7: int = 194
    # KEYCODE_BUTTON_8	Key code constant: Generic Game Pad Button 	8
    KEYCODE_BUTTON_8: int = 195
    # KEYCODE_BUTTON_9	Key code constant: Generic Game Pad Button 	9
    KEYCODE_BUTTON_9: int = 196
    # KEYCODE_BUTTON_A	Key code constant: A Button key.
    KEYCODE_BUTTON_A: int = 96
    # KEYCODE_BUTTON_B	Key code constant: B Button key.
    KEYCODE_BUTTON_B: int = 97
    # KEYCODE_BUTTON_C	Key code constant: C Button key.
    KEYCODE_BUTTON_C: int = 98
    # KEYCODE_BUTTON_L1	Key code constant: L1 Button key.
    KEYCODE_BUTTON_L1: int = 102
    # KEYCODE_BUTTON_L2	Key code constant: L2 Button key.
    KEYCODE_BUTTON_L2: int = 104
    # KEYCODE_BUTTON_MODE	Key code constant: Mode Button key.
    KEYCODE_BUTTON_MODE: int = 110
    # KEYCODE_BUTTON_R1	Key code constant: R1 Button key.
    KEYCODE_BUTTON_R1: int = 103
    # KEYCODE_BUTTON_R2	Key code constant: R2 Button key.
    KEYCODE_BUTTON_R2: int = 105
    # KEYCODE_BUTTON_SELECT	Key code constant: Select Button key.
    KEYCODE_BUTTON_SELECT: int = 109
    # KEYCODE_BUTTON_START	Key code constant: Start Button key.
    KEYCODE_BUTTON_START: int = 108
    # KEYCODE_BUTTON_THUMBL	Key code constant: Left Thumb Button key.
    KEYCODE_BUTTON_THUMBL: int = 106
    # KEYCODE_BUTTON_THUMBR	Key code constant: Right Thumb Button key.
    KEYCODE_BUTTON_THUMBR: int = 107
    # KEYCODE_BUTTON_X	Key code constant: X Button key.
    KEYCODE_BUTTON_X: int = 99
    # KEYCODE_BUTTON_Y	Key code constant: Y Button key.
    KEYCODE_BUTTON_Y: int = 100
    # KEYCODE_BUTTON_Z	Key code constant: Z Button key.
    KEYCODE_BUTTON_Z: int = 101
    KEYCODE_C: int = 31  # KEYCODE_C	Key code constant: 'C' key.
    # KEYCODE_CALCULATOR	Key code constant: Calculator special function key.
    KEYCODE_CALCULATOR: int = 210
    # KEYCODE_CALENDAR	Key code constant: Calendar special function key.
    KEYCODE_CALENDAR: int = 208
    KEYCODE_CALL: int = 5  # KEYCODE_CALL	Key code constant: Call key.
    KEYCODE_CAMERA: int = 27  # KEYCODE_CAMERA	Key code constant: Camera key.
    # KEYCODE_CAPS_LOCK	Key code constant: Caps Lock key.
    KEYCODE_CAPS_LOCK: int = 115
    # KEYCODE_CAPTIONS	Key code constant: Toggle captions key.
    KEYCODE_CAPTIONS: int = 175
    # KEYCODE_CHANNEL_DOWN	Key code constant: Channel down key.
    KEYCODE_CHANNEL_DOWN: int = 167
    # KEYCODE_CHANNEL_UP	Key code constant: Channel up key.
    KEYCODE_CHANNEL_UP: int = 166
    KEYCODE_CLEAR: int = 28  # KEYCODE_CLEAR	Key code constant: Clear key.
    KEYCODE_COMMA: int = 55  # KEYCODE_COMMA	Key code constant: ',' key.
    # KEYCODE_CONTACTS	Key code constant: Contacts special function key.
    KEYCODE_CONTACTS: int = 207
    # KEYCODE_CTRL_LEFT	Key code constant: Left Control modifier key.
    KEYCODE_CTRL_LEFT: int = 113
    # KEYCODE_CTRL_RIGHT	Key code constant: Right Control modifier key.
    KEYCODE_CTRL_RIGHT: int = 114
    KEYCODE_D: int = 32  # KEYCODE_D	Key code constant: 'D' key.
    KEYCODE_DEL: int = 67  # KEYCODE_DEL	Key code constant: Backspace key.
    # KEYCODE_DPAD_CENTER	Key code constant: Directional Pad Center key.
    KEYCODE_DPAD_CENTER: int = 23
    # KEYCODE_DPAD_DOWN	Key code constant: Directional Pad Down key.
    KEYCODE_DPAD_DOWN: int = 20
    # KEYCODE_DPAD_LEFT	Key code constant: Directional Pad Left key.
    KEYCODE_DPAD_LEFT: int = 21
    # KEYCODE_DPAD_RIGHT	Key code constant: Directional Pad Right key.
    KEYCODE_DPAD_RIGHT: int = 22
    # KEYCODE_DPAD_UP	Key code constant: Directional Pad Up key.
    KEYCODE_DPAD_UP: int = 19
    KEYCODE_DVR: int = 173  # KEYCODE_DVR	Key code constant: DVR key.
    KEYCODE_E: int = 33  # KEYCODE_E	Key code constant: 'E' key.
    # KEYCODE_EISU	Key code constant: Japanese alphanumeric key.
    KEYCODE_EISU: int = 212
    # KEYCODE_ENDCALL	Key code constant: End Call key.
    KEYCODE_ENDCALL: int = 6
    KEYCODE_ENTER: int = 66  # KEYCODE_ENTER	Key code constant: Enter key.
    # KEYCODE_ENVELOPE	Key code constant: Envelope special function key.
    KEYCODE_ENVELOPE: int = 65
    KEYCODE_EQUALS: int = 70  # KEYCODE_EQUALS	Key code constant: '=' key.
    KEYCODE_ESCAPE: int = 111  # KEYCODE_ESCAPE	Key code constant: Escape key.
    # KEYCODE_EXPLORER	Key code constant: Explorer special function key.
    KEYCODE_EXPLORER: int = 64
    KEYCODE_F: int = 34  # KEYCODE_F	Key code constant: 'F' key.
    KEYCODE_F1: int = 131  # KEYCODE_F1	Key code constant: F1 key.
    KEYCODE_F10: int = 140  # KEYCODE_F10	Key code constant: F10 key.
    KEYCODE_F11: int = 141  # KEYCODE_F11	Key code constant: F11 key.
    KEYCODE_F12: int = 142  # KEYCODE_F12	Key code constant: F12 key.
    KEYCODE_F2: int = 132  # KEYCODE_F2	Key code constant: F2 key.
    KEYCODE_F3: int = 133  # KEYCODE_F3	Key code constant: F3 key.
    KEYCODE_F4: int = 134  # KEYCODE_F4	Key code constant: F4 key.
    KEYCODE_F5: int = 135  # KEYCODE_F5	Key code constant: F5 key.
    KEYCODE_F6: int = 136  # KEYCODE_F6	Key code constant: F6 key.
    KEYCODE_F7: int = 137  # KEYCODE_F7	Key code constant: F7 key.
    KEYCODE_F8: int = 138  # KEYCODE_F8	Key code constant: F8 key.
    KEYCODE_F9: int = 139  # KEYCODE_F9	Key code constant: F9 key.
    # KEYCODE_FOCUS	Key code constant: Camera Focus key.
    KEYCODE_FOCUS: int = 80
    # KEYCODE_FORWARD	Key code constant: Forward key.
    KEYCODE_FORWARD: int = 125
    # KEYCODE_FORWARD_DEL	Key code constant: Forward Delete key.
    KEYCODE_FORWARD_DEL: int = 112
    # KEYCODE_FUNCTION	Key code constant: Function modifier key.
    KEYCODE_FUNCTION: int = 119
    KEYCODE_G: int = 35  # KEYCODE_G	Key code constant: 'G' key.
    # KEYCODE_GRAVE	Key code constant: '`' (backtick) key.
    KEYCODE_GRAVE: int = 68
    KEYCODE_GUIDE: int = 172  # KEYCODE_GUIDE	Key code constant: Guide key.
    KEYCODE_H: int = 36  # KEYCODE_H	Key code constant: 'H' key.
    # KEYCODE_HEADSETHOOK	Key code constant: Headset Hook key.
    KEYCODE_HEADSETHOOK: int = 79
    # KEYCODE_HENKAN	Key code constant: Japanese conversion key.
    KEYCODE_HENKAN: int = 214
    KEYCODE_HOME: int = 3  # KEYCODE_HOME	Key code constant: Home key.
    KEYCODE_I: int = 37  # KEYCODE_I	Key code constant: 'I' key.
    KEYCODE_INFO: int = 165  # KEYCODE_INFO	Key code constant: Info key.
    KEYCODE_INSERT: int = 124  # KEYCODE_INSERT	Key code constant: Insert key.
    KEYCODE_J: int = 38  # KEYCODE_J	Key code constant: 'J' key.
    KEYCODE_K: int = 39  # KEYCODE_K	Key code constant: 'K' key.
    # KEYCODE_KANA	Key code constant: Japanese kana key.
    KEYCODE_KANA: int = 218
    # KEYCODE_KATAKANA_HIRAGANA	Key code constant: Japanese katakana / hiragana key.
    KEYCODE_KATAKANA_HIRAGANA: int = 215
    KEYCODE_L: int = 40  # KEYCODE_L	Key code constant: 'L' key.
    # KEYCODE_LANGUAGE_SWITCH	Key code constant: Language Switch key.
    KEYCODE_LANGUAGE_SWITCH: int = 204
    # KEYCODE_LEFT_BRACKET	Key code constant: '[' key.
    KEYCODE_LEFT_BRACKET: int = 71
    KEYCODE_M: int = 41  # KEYCODE_M	Key code constant: 'M' key.
    # KEYCODE_MANNER_MODE	Key code constant: Manner Mode key.
    KEYCODE_MANNER_MODE: int = 205
    # KEYCODE_MEDIA_CLOSE	Key code constant: Close media key.
    KEYCODE_MEDIA_CLOSE: int = 128
    # KEYCODE_MEDIA_EJECT	Key code constant: Eject media key.
    KEYCODE_MEDIA_EJECT: int = 129
    # KEYCODE_MEDIA_FAST_FORWARD	Key code constant: Fast Forward media key.
    KEYCODE_MEDIA_FAST_FORWARD: int = 90
    # KEYCODE_MEDIA_NEXT	Key code constant: Play Next media key.
    KEYCODE_MEDIA_NEXT: int = 87
    # KEYCODE_MEDIA_PAUSE	Key code constant: Pause media key.
    KEYCODE_MEDIA_PAUSE: int = 127
    # KEYCODE_MEDIA_PLAY	Key code constant: Play media key.
    KEYCODE_MEDIA_PLAY: int = 126
    # KEYCODE_MEDIA_PLAY_PAUSE	Key code constant: Play/Pause media key.
    KEYCODE_MEDIA_PLAY_PAUSE: int = 85
    # KEYCODE_MEDIA_PREVIOUS	Key code constant: Play Previous media key.
    KEYCODE_MEDIA_PREVIOUS: int = 88
    # KEYCODE_MEDIA_RECORD	Key code constant: Record media key.
    KEYCODE_MEDIA_RECORD: int = 130
    # KEYCODE_MEDIA_REWIND	Key code constant: Rewind media key.
    KEYCODE_MEDIA_REWIND: int = 89
    # KEYCODE_MEDIA_STOP	Key code constant: Stop media key.
    KEYCODE_MEDIA_STOP: int = 86
    KEYCODE_MENU: int = 82  # KEYCODE_MENU	Key code constant: Menu key.
    # KEYCODE_META_LEFT	Key code constant: Left Meta modifier key.
    KEYCODE_META_LEFT: int = 117
    # KEYCODE_META_RIGHT	Key code constant: Right Meta modifier key.
    KEYCODE_META_RIGHT: int = 118
    KEYCODE_MINUS: int = 69  # KEYCODE_MINUS	Key code constant: '-'.
    # KEYCODE_MOVE_END	Key code constant: End Movement key.
    KEYCODE_MOVE_END: int = 123
    # KEYCODE_MOVE_HOME	Key code constant: Home Movement key.
    KEYCODE_MOVE_HOME: int = 122
    # KEYCODE_MUHENKAN	Key code constant: Japanese non-conversion key.
    KEYCODE_MUHENKAN: int = 213
    # KEYCODE_MUSIC	Key code constant: Music special function key.
    KEYCODE_MUSIC: int = 209
    KEYCODE_MUTE: int = 91  # KEYCODE_MUTE	Key code constant: Mute key.
    KEYCODE_N: int = 42  # KEYCODE_N	Key code constant: 'N' key.
    # KEYCODE_NOTIFICATION	Key code constant: Notification key.
    KEYCODE_NOTIFICATION: int = 83
    # KEYCODE_NUM	Key code constant: Number modifier key.
    KEYCODE_NUM: int = 78
    # KEYCODE_NUMPAD_0	Key code constant: Numeric keypad '0' key.
    KEYCODE_NUMPAD_0: int = 144
    # KEYCODE_NUMPAD_1	Key code constant: Numeric keypad '1' key.
    KEYCODE_NUMPAD_1: int = 145
    # KEYCODE_NUMPAD_2	Key code constant: Numeric keypad '2' key.
    KEYCODE_NUMPAD_2: int = 146
    # KEYCODE_NUMPAD_3	Key code constant: Numeric keypad '3' key.
    KEYCODE_NUMPAD_3: int = 147
    # KEYCODE_NUMPAD_4	Key code constant: Numeric keypad '4' key.
    KEYCODE_NUMPAD_4: int = 148
    # KEYCODE_NUMPAD_5	Key code constant: Numeric keypad '5' key.
    KEYCODE_NUMPAD_5: int = 149
    # KEYCODE_NUMPAD_6	Key code constant: Numeric keypad '6' key.
    KEYCODE_NUMPAD_6: int = 150
    # KEYCODE_NUMPAD_7	Key code constant: Numeric keypad '7' key.
    KEYCODE_NUMPAD_7: int = 151
    # KEYCODE_NUMPAD_8	Key code constant: Numeric keypad '8' key.
    KEYCODE_NUMPAD_8: int = 152
    # KEYCODE_NUMPAD_9	Key code constant: Numeric keypad '9' key.
    KEYCODE_NUMPAD_9: int = 153
    # KEYCODE_NUMPAD_ADD	Key code constant: Numeric keypad '+' key (for addition).
    KEYCODE_NUMPAD_ADD: int = 157
    # KEYCODE_NUMPAD_COMMA	Key code constant: Numeric keypad ',' key (for decimals or digit grouping).
    KEYCODE_NUMPAD_COMMA: int = 159
    # KEYCODE_NUMPAD_DIVIDE	Key code constant: Numeric keypad '/' key (for division).
    KEYCODE_NUMPAD_DIVIDE: int = 154
    # KEYCODE_NUMPAD_DOT	Key code constant: Numeric keypad '.' key (for decimals or digit grouping).
    KEYCODE_NUMPAD_DOT: int = 158
    # KEYCODE_NUMPAD_ENTER	Key code constant: Numeric keypad Enter key.
    KEYCODE_NUMPAD_ENTER: int = 160
    # KEYCODE_NUMPAD_EQUALS	Key code constant: Numeric keypad '=' key.
    KEYCODE_NUMPAD_EQUALS: int = 161
    # KEYCODE_NUMPAD_LEFT_PAREN	Key code constant: Numeric keypad '(' key.
    KEYCODE_NUMPAD_LEFT_PAREN: int = 162
    # KEYCODE_NUMPAD_MULTIPLY	Key code constant: Numeric keypad '*' key (for multiplication).
    KEYCODE_NUMPAD_MULTIPLY: int = 155
    # KEYCODE_NUMPAD_RIGHT_PAREN	Key code constant: Numeric keypad ')' key.
    KEYCODE_NUMPAD_RIGHT_PAREN: int = 163
    # KEYCODE_NUMPAD_SUBTRACT	Key code constant: Numeric keypad '-' key (for subtraction).
    KEYCODE_NUMPAD_SUBTRACT: int = 156
    # KEYCODE_NUM_LOCK	Key code constant: Num Lock key.
    KEYCODE_NUM_LOCK: int = 143
    KEYCODE_O: int = 43  # KEYCODE_O	Key code constant: 'O' key.
    KEYCODE_P: int = 44  # KEYCODE_P	Key code constant: 'P' key.
    # KEYCODE_PAGE_DOWN	Key code constant: Page Down key.
    KEYCODE_PAGE_DOWN: int = 93
    # KEYCODE_PAGE_UP	Key code constant: Page Up key.
    KEYCODE_PAGE_UP: int = 92
    KEYCODE_PERIOD: int = 56  # KEYCODE_PERIOD	Key code constant: '.' key.
    # KEYCODE_PICTSYMBOLS	Key code constant: Picture Symbols modifier key.
    KEYCODE_PICTSYMBOLS: int = 94
    KEYCODE_PLUS: int = 81  # KEYCODE_PLUS	Key code constant: '+' key.
    KEYCODE_POUND: int = 18  # KEYCODE_POUND	Key code constant: '	' key.
    KEYCODE_POWER: int = 26  # KEYCODE_POWER	Key code constant: Power key.
    # KEYCODE_PROG_BLUE	Key code constant: Blue "programmable" key.
    KEYCODE_PROG_BLUE: int = 186
    # KEYCODE_PROG_GREEN	Key code constant: Green "programmable" key.
    KEYCODE_PROG_GREEN: int = 184
    # KEYCODE_PROG_RED	Key code constant: Red "programmable" key.
    KEYCODE_PROG_RED: int = 183
    # KEYCODE_PROG_YELLOW	Key code constant: Yellow "programmable" key.
    KEYCODE_PROG_YELLOW: int = 185
    KEYCODE_Q: int = 45  # KEYCODE_Q	Key code constant: 'Q' key.
    KEYCODE_R: int = 46  # KEYCODE_R	Key code constant: 'R' key.
    # KEYCODE_RIGHT_BRACKET	Key code constant: ']' key.
    KEYCODE_RIGHT_BRACKET: int = 72
    KEYCODE_RO: int = 217  # KEYCODE_RO	Key code constant: Japanese Ro key.
    KEYCODE_S: int = 47  # KEYCODE_S	Key code constant: 'S' key.
    # KEYCODE_SCROLL_LOCK	Key code constant: Scroll Lock key.
    KEYCODE_SCROLL_LOCK: int = 116
    KEYCODE_SEARCH: int = 84  # KEYCODE_SEARCH	Key code constant: Search key.
    # KEYCODE_SEMICOLON	Key code constant: ';' key.
    KEYCODE_SEMICOLON: int = 74
    # KEYCODE_SETTINGS	Key code constant: Settings key.
    KEYCODE_SETTINGS: int = 176
    # KEYCODE_SHIFT_LEFT	Key code constant: Left Shift modifier key.
    KEYCODE_SHIFT_LEFT: int = 59
    # KEYCODE_SHIFT_RIGHT	Key code constant: Right Shift modifier key.
    KEYCODE_SHIFT_RIGHT: int = 60
    KEYCODE_SLASH: int = 76  # KEYCODE_SLASH	Key code constant: '/' key.
    # KEYCODE_SOFT_LEFT	Key code constant: Soft Left key.
    KEYCODE_SOFT_LEFT: int = 1
    # KEYCODE_SOFT_RIGHT	Key code constant: Soft Right key.
    KEYCODE_SOFT_RIGHT: int = 2
    KEYCODE_SPACE: int = 62  # KEYCODE_SPACE	Key code constant: Space key.
    KEYCODE_STAR: int = 17  # KEYCODE_STAR	Key code constant: '*' key.
    # KEYCODE_STB_INPUT	Key code constant: Set-top-box input key.
    KEYCODE_STB_INPUT: int = 180
    # KEYCODE_STB_POWER	Key code constant: Set-top-box power key.
    KEYCODE_STB_POWER: int = 179
    # KEYCODE_SWITCH_CHARSET	Key code constant: Switch Charset modifier key.
    KEYCODE_SWITCH_CHARSET: int = 95
    # KEYCODE_SYM	Key code constant: Symbol modifier key.
    KEYCODE_SYM: int = 63
    # KEYCODE_SYSRQ	Key code constant: System Request / Print Screen key.
    KEYCODE_SYSRQ: int = 120
    KEYCODE_T: int = 48  # KEYCODE_T	Key code constant: 'T' key.
    KEYCODE_TAB: int = 61  # KEYCODE_TAB	Key code constant: Tab key.
    KEYCODE_TV: int = 170  # KEYCODE_TV	Key code constant: TV key.
    # KEYCODE_TV_INPUT	Key code constant: TV input key.
    KEYCODE_TV_INPUT: int = 178
    # KEYCODE_TV_POWER	Key code constant: TV power key.
    KEYCODE_TV_POWER: int = 177
    KEYCODE_U: int = 49  # KEYCODE_U	Key code constant: 'U' key.
    # KEYCODE_UNKNOWN	Key code constant: Unknown key code.
    KEYCODE_UNKNOWN: int = 0
    KEYCODE_V: int = 50  # KEYCODE_V	Key code constant: 'V' key.
    # KEYCODE_VOLUME_DOWN	Key code constant: Volume Down key.
    KEYCODE_VOLUME_DOWN: int = 25
    # KEYCODE_VOLUME_MUTE	Key code constant: Volume Mute key.
    KEYCODE_VOLUME_MUTE: int = 164
    # KEYCODE_VOLUME_UP	Key code constant: Volume Up key.
    KEYCODE_VOLUME_UP: int = 24
    KEYCODE_W: int = 51  # KEYCODE_W	Key code constant: 'W' key.
    KEYCODE_WINDOW: int = 171  # KEYCODE_WINDOW	Key code constant: Window key.
    KEYCODE_X: int = 52  # KEYCODE_X	Key code constant: 'X' key.
    KEYCODE_Y: int = 53  # KEYCODE_Y	Key code constant: 'Y' key.
    KEYCODE_YEN: int = 216  # KEYCODE_YEN	Key code constant: Japanese Yen key.
    KEYCODE_Z: int = 54  # KEYCODE_Z	Key code constant: 'Z' key.
    # KEYCODE_ZENKAKU_HANKAKU	Key code constant: Japanese full-width / half-width key.
    KEYCODE_ZENKAKU_HANKAKU: int = 211
    # KEYCODE_ZOOM_IN	Key code constant: Zoom in key.
    KEYCODE_ZOOM_IN: int = 168
    # KEYCODE_ZOOM_OUT	Key code constant: Zoom out key.
    KEYCODE_ZOOM_OUT: int = 169

    # MAX_KEYCODE	This constant was deprecated in API level 3. There are now more than MAX_KEYCODE keycodes. Use getMaxKeyCode() instead.
    MAX_KEYCODE: int = 84
    # META_ALT_LEFT_ON This mask is is used to check whether the left ALT meta key is pressed.
    META_ALT_LEFT_ON: int = 16
    # META_ALT_MASK		This mask is is a combination of META_ALT_ON, META_ALT_LEFT_ON and META_ALT_RIGHT_ON.
    META_ALT_MASK: int = 50
    # META_ALT_ON This mask is is used to check whether one of the ALT meta keys is pressed.
    META_ALT_ON: int = 2
    # META_ALT_RIGHT_ON This mask is is used to check whether the right the ALT meta key is pressed.
    META_ALT_RIGHT_ON: int = 32
    # META_CAPS_LOCK_ON This mask is is used to check whether the CAPS LOCK meta key is on.
    META_CAPS_LOCK_ON: int = 1048576
    # META_CTRL_LEFT_ON This mask is is used to check whether the left CTRL meta key is pressed.
    META_CTRL_LEFT_ON: int = 8192
    # META_CTRL_MASK		This mask is is a combination of META_CTRL_ON, META_CTRL_LEFT_ON and META_CTRL_RIGHT_ON.
    META_CTRL_MASK: int = 28672
    # META_CTRL_ON This mask is is used to check whether one of the CTRL meta keys is pressed.
    META_CTRL_ON: int = 4096
    # META_CTRL_RIGHT_ON This mask is is used to check whether the right CTRL meta key is pressed.
    META_CTRL_RIGHT_ON: int = 16384
    # META_FUNCTION_ON This mask is is used to check whether the FUNCTION meta key is pressed.
    META_FUNCTION_ON: int = 8
    # META_META_LEFT_ON This mask is is used to check whether the left META meta key is pressed.
    META_META_LEFT_ON: int = 131072
    # META_META_MASK		This mask is is a combination of META_META_ON, META_META_LEFT_ON and META_META_RIGHT_ON.
    META_META_MASK: int = 458752
    # META_META_ON This mask is is used to check whether one of the META meta keys is pressed.
    META_META_ON: int = 65536
    # META_META_RIGHT_ON This mask is is used to check whether the right META meta key is pressed.
    META_META_RIGHT_ON: int = 262144
    # META_NUM_LOCK_ON This mask is is used to check whether the NUM LOCK meta key is on.
    META_NUM_LOCK_ON: int = 2097152
    # META_SCROLL_LOCK_ON This mask is is used to check whether the SCROLL LOCK meta key is on.
    META_SCROLL_LOCK_ON: int = 4194304
    # META_SHIFT_LEFT_ON This mask is is used to check whether the left SHIFT meta key is pressed.
    META_SHIFT_LEFT_ON: int = 64
    # META_SHIFT_MASK		This mask is is a combination of META_SHIFT_ON, META_SHIFT_LEFT_ON and META_SHIFT_RIGHT_ON.
    META_SHIFT_MASK: int = 193
    # META_SHIFT_ON This mask is is used to check whether one of the SHIFT meta keys is pressed.
    META_SHIFT_ON: int = 1
    # META_SHIFT_RIGHT_ON This mask is is used to check whether the right SHIFT meta key is pressed.
    META_SHIFT_RIGHT_ON: int = 128
    # META_SYM_ON This mask is is used to check whether the SYM meta key is pressed.
    META_SYM_ON: int = 4



from select import select
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver import ActionChains
from selenium.webdriver.common.alert import Alert
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support.select import Select
from pyallied.web.webWaits import customwebDriverwait

class frameAndWindow(customwebDriverwait):

    def __init__(self, driver):
        #self.driver = driver
        super().__init__(driver)
        self.driver=driver
    def Get_Window_handles(self):
        try:
            return self.driver.window_handles
        except Exception as error:
            raise error
    def Get_window_Position(self):
        try:
            return self.driver.get_window_position()
        except Exception as error:
            raise error
    def Get_window_size(self):
        try:
            return self.driver.get_window_size()
        except Exception as error:
            raise error
    def Minimize_window(self):
        try:
            self.driver.minimize_window()
        except Exception as error:
            raise error
    def Maximize_window(self):
        try:
            self.driver.maximize_window()
        except Exception as error:
            raise error            

    # switch to window
    # ex: <a href="somewhere.html" target="windowName">Click here to open a new window</a>

    def switchToWindowUsingName(self, windowName):
        try:
            self.driver.switch_to.window(windowName)
        except Exception as error:
            raise error
    def switchtoWindowUsingHandle(self, windowNumber):
        try:
            current_window = self.driver.current_window_handle
            ListOfHandles = self.driver.window_handles
            if(current_window != ListOfHandles[windowNumber]):
                self.driver.switch_to.window(ListOfHandles[windowNumber])
        except Exception as error:
            raise error                
    # frame switch, It’s possible to access subframes by separating the path with a dot,
    # and you can specify the frame by its index too. That is: driver.switch_to_frame("frameName.0.child")
    # would go to the frame named “child” of the first subframe of the frame called “frameName”. All frames are evaluated as if from *top*.

    def switchToFrame(self, xpath):
        try:
            return super().WaitFor_frame_to_be_available_and_switch_to_it(xpath)
        except Exception as error:
            raise error        
    # Once we are done with working on frames, we will have to come back to the parent
    # frame which can be done using:

    def switchToParentFrame(self):
        try:
            self.driver.switch_to.default_content()
        except Exception as error:
            raise error
    def switch_To_Frame_ByXpath(self,frameXpath):
        try:
            #self.driver.presence_of_element_located(frameXpath)
            if(self.driver.presence_of_element_located(frameXpath)) :
                element=self.driver.find_element(By.XPATH, frameXpath)
                self.driver.switch_to.frame(element)
        except Exception as error:
            raise error
    def switch_To_Frame_ByIndex(self,index:int):
        try:

            self.driver.switch_to.frame(index)
        except Exception as error:
            raise error
    def switch_To_Frame_ByName(self,name):
        try:
            self.driver.switch_to.frame(name)
        except Exception as error:
            raise error       
    def switch_To_Parent_Frame(self):
        try:

            self.driver.switch_to.parent_frame()
        except Exception as error:
            raise error                                                    
    # navigating



from pyallied.web.webWaits import customwebDriverwait
import os
import base64
from PIL import Image
class firefox(customwebDriverwait):
    def __init__(self, driver):
        super().__init__(driver)
        self.driver = driver
    def Uninstall_addon(self,identifier):
        try:
            self.driver.uninstall_addon(identifier)
        except Exception as error:
            raise error
    '''
    Installs Firefox addon.

    Returns identifier of installed addon. This identifier can later be used to uninstall addon.

    Parameters:	
    path – Absolute path to the addon that will be installed.

    Usage:	
    driver.install_addon('/path/to/firebug.xpi')
    '''
    def Install_addon(self,path, temporary=False):
        try:
            self.driver.install_addon(path)
        except Exception as error:
            raise error
    
    def Set_context(self,context):
        try:
            self.driver.set_context(context)
        except Exception as error:
            raise error
    def Get_firefox_profile(self,context):
        try:
           return self.driver.firefox_profile
        except Exception as error:
            raise error
    '''def is_headless(self):
        #if(self.driver.capabilities['browser_name']=="firefox"):
           return self.driver.headless
        else:
            raise "is headless? is allowed only on firefox" '''
    def Add_extension(self,extension='webdriver.xpi'):
        try:
           return self.driver.firefox_profile.add_extension(extension)
        except Exception as error:
            raise error
    '''sets the preference that we want in the profile.'''        
    def Set_preference(self,key, value):
        try:
            self.driver.firefox_profile.set_preference(key, value)
        except Exception as error:
            raise error      
    def get_default_capabilities(self):
        try:
           return self.driver.firefox_profile.default_capabilities
        except Exception as error:
            raise error  
    def Update_capabilities(self):
        try:
           return self.driver.firefox_profile.update_preferences()
        except Exception as error:
            raise error                          
    def get_binary_instance(self):
        try:
           return self.driver.binary()
        except Exception as error:
            raise error   
    def Set_accept_untrusted_certs(self):
        try:
           return self.driver.firefox_profile.accept_untrusted_certs
        except Exception as error:
            raise error
    def Set_assume_untrusted_cert_issuer(self):
        try:
           return self.driver.firefox_profile.assume_untrusted_cert_issuer
        except Exception as error:
            raise error
    def Get_encoded_Profile(self):
        try:
           return self.driver.firefox_profile.encoded
        except Exception as error:
            raise error
    def Get_Profile_path(self):
        try:
           return self.driver.firefox_profile.path
        except Exception as error:
            raise error
    def Get_Profile_DEFAULT_PREFERENCES(self):
        try:
           return self.driver.firefox_profile.DEFAULT_PREFERENCES 
        except Exception as error:
            raise error
    def Get_Profile_ANONYMOUS_PROFILE_NAME(self):
        try:
           return self.driver.firefox_profile.ANONYMOUS_PROFILE_NAME 
        except Exception as error:
            raise error
    def Get_Profile_port(self):
        try:
           return self.driver.firefox_profile.port 
        except Exception as error:
            raise error
    def Add_command_line_options(self,*args):
        try:
           return self.driver.binary.add_command_line_options(*args) 
        except Exception as error:
            raise error
    def kill_Binary_instance(self,*args):
        try:
           return self.driver.binary.kill() 
        except Exception as error:
            raise error
    def Launch_browser(self,profile, timeout=30):
        try:
           return self.driver.binary.launch_browser(profile, timeout=30) 
        except Exception as error:
            raise error 
    '''
        Gets the full document screenshot of the current window as a base64 encoded string
        which is useful in embedded images in HTML.

        Usage:	
        driver.get_full_page_screenshot_as_base64()
    '''
    def get_fullpage_screenshot_as_base64(self,name):
        try:
            with open(self.__web_Screenshot_Location()+"/"+name+".jpg", "wb") as fh:
                fh.write(base64.urlsafe_b64decode(self.driver.get_full_page_screenshot_as_base64()))             
             
        except Exception as error:
            raise error    

    '''
        Saves a full document screenshot of the current window to a PNG image file. Returns
        False if there is any IOError, else returns True. Use full paths in your filename.
        Args:	
        filename: The full path you wish to save your screenshot to. This should end with a .png extension.
        Usage:	
        driver.get_full_page_screenshot_as_file('/Screenshots/foo.png')
    '''
    def get_fullpage_screenshot_as_File(self,filename:str):
        try:
            return self.driver.get_full_page_screenshot_as_file(self.__web_Screenshot_Location()+"/"+filename+".png") 
        except Exception as error:
            raise error       
    '''get_full_page_screenshot_as_png() → str
        Gets the full document screenshot of the current window as a binary data.

        Usage:	
        driver.get_full_page_screenshot_as_png()
    '''
    def get_fullpage_screenshot_as_PNG(self,name):
        try:
            result_File=self.__web_Screenshot_Location()+"/"+name+".png"
            with open(result_File, "wb") as fh:
                fh.write(self.driver.get_full_page_screenshot_as_png())

             
        except Exception as error:
            raise error    

    ''' 
        Saves a screenshot of the current window to a PNG image file. Returns
        False if there is any IOError, else returns True. Use full paths in your filename.
        Args:	
        filename: The full path you wish to save your screenshot to. This should end with a .png extension.
        Usage:	
        driver.save_screenshot('/Screenshots/foo.png')
    '''
    def screenshot_save_full_page_screenshot(self,filename):
        try:
            return self.driver.save_screenshot(filename)
        except Exception as error:
            raise error
    def __web_Screenshot_Location(self):
        try:
            dir_name=os.getcwd()+"/page_Screenshots"
            os.makedirs(dir_name, exist_ok=True)
            return dir_name
        except FileExistsError:
            pass                
                                                                                                                                                                                                                        


from select import select
from xml.dom.minidom import Element
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver import ActionChains
from selenium.webdriver.common.alert import Alert
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support.select import Select
from pyallied.web.webWaits import customwebDriverwait
from selenium.webdriver.remote.webdriver import WebDriver
from selenium.webdriver.common.actions.wheel_input import ScrollOrigin
import os
import base64
from PIL import Image

class common(customwebDriverwait):

    def __init__(self, driver):
        super().__init__(driver)
        self.driver = driver

    def findElementsBy(self, xpath):
        try:
            if(super().WaitFor_presence_of_all_elements_located(xpath)):
                return self.driver.find_elements(By.XPATH, xpath)
            else:
                self.__raise_element_not_present_exception(xpath)    
        except Exception as error:
            raise error

        # self.driver.presence_of_all_elements_located(xpath)
        # self.driver.visibility_of_element_located(xpath)
        
        

    def findElementBy(self, xpath):
        try:
            elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
            elementVisible = super().WaitFor_VisibilityOf_Element_Located(xpath)
            if elementPresense :
                if elementVisible:
                    return elementVisible
                else:
                    self.__raise_element_not_visible_exception(xpath) 
            else:
                self.__raise_element_not_present_exception(xpath)           

        except Exception as error:
            raise error        

    def click(self, xpath):
        try:
            elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
            elementVisible = super().WaitFor_VisibilityOf_Element_Located(xpath)
            if(elementPresense):
                if(elementVisible):
                    elementVisible.click()
                else:
                    self.__raise_element_not_visible_exception(xpath) 
            else:
                self.__raise_element_not_present_exception(xpath)                     
        except Exception as error:
            raise error            

    def clear(self, xpath):
        try:
            elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
            elementVisible = super().WaitFor_VisibilityOf_Element_Located(xpath)
            if(elementPresense):
                if(elementVisible):
                    elementVisible.clear()
                else:
                    self.__raise_element_not_visible_exception(xpath) 
            else:
                self.__raise_element_not_present_exception(xpath)                     
        except Exception as error:
            raise error              
    
    
    def isClickable(self, xpath, index=None):
        # self.driver.visibility_of_element_located(xpath)
        try:
            elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
            elementVisible = super().WaitFor_VisibilityOf_Element_Located(xpath)
            if(elementPresense):
                if(elementVisible):
            #return WebDriverWait(self.driver, 10).until(EC.presence_of_element_located((xpath)))
                    return elementVisible
                else:
                    self.__raise_element_not_visible_exception(xpath) 
            else:
                self.__raise_element_not_present_exception(xpath)                     
        except Exception as error:
            raise error    
    
    '''
    Simulates typing into the element.

    Args:	
    value - A string for typing, or setting form fields. For setting file inputs, this could be a local file path.
    Use this to send simple key events or to fill out form fields:
    form_textfield = driver.find_element(By.NAME, 'username')
                    form_textfield.send_keys("admin")
    This can also be used to set file inputs.

    file_input = driver.find_element(By.NAME, 'profilePic')
    file_input.send_keys("path/to/profilepic.gif")
    # Generally it's better to wrap the file path in one of the methods
    # in os.path to return the actual path to support cross OS testing.
    # file_input.send_keys(os.path.abspath("path/to/profilepic.gif"))
    '''

    def fillField(self, xpath, text):
        try:
            elementPresence = super().WaitFor_PresenseOf_Element_Located(xpath)
            elementVisibility = super().WaitFor_VisibilityOf_Element_Located(xpath)
            if(elementPresence):
                if(elementVisibility):
                    elementVisibility.send_keys(text)
                else:
                    self.__raise_element_not_visible_exception(xpath) 
            else:
                self.__raise_element_not_present_exception(xpath)                     
        except Exception as error:
            raise error

    def navigateto(self, urlstring):
        try:
            self.driver.get(urlstring)
        except Exception as error:
            raise error
    '''
        Gets the given attribute or property of the element.

        This method will first try to return the value of a property with the given name. If a property with that name doesn’t exist, it returns the value of the attribute with the same name. If there’s no attribute with that name, None is returned.

        Values which are considered truthy, that is equals “true” or “false”, are returned as booleans. All other non-None values are returned as strings. For attributes or properties which do not exist, None is returned.

        To obtain the exact value of the attribute or property, use get_dom_attribute() or get_property() methods respectively.

        Args:	
        name - Name of the attribute/property to retrieve.
        Example:

        # Check if the "active" CSS class is applied to an element.
        is_active = "active" in target_element.get_attribute("class")
    '''
    def getAttribute(self, xpath,AttributeName):
        try:
            elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
            #elementVisible = super().WaitFor_VisibilityOf_Element_Located(xpath)
            if(elementPresense):
                #if(elementVisible):
                    return elementPresense.get_attribute(AttributeName) 
            else:
                self.__raise_element_not_present_exception(xpath)                     
        except Exception as error:
            raise error                  
    
    '''
        Gets the given attribute of the element. Unlike get_attribute(), this method only returns 
        attributes declared in the element’s HTML markup.

        Args:	
        name - Name of the attribute to retrieve.
        Usage:	
        text_length = target_element.get_dom_attribute("class")
    '''
    def getDomAttribute(self, xpath,AttributeName):
        try:
            elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
            #elementVisible = super().WaitFor_VisibilityOf_Element_Located(xpath)
            if(elementPresense):
                #if(elementVisible):
                    return elementPresense.get_dom_attribute(AttributeName)
            else:
                self.__raise_element_not_present_exception(xpath)                    
        except Exception as error:
            raise error


    '''
        Gets the given property of the element.

        Args:	
        name - Name of the property to retrieve.
        Usage:	
        text_length = target_element.get_property("text_length")
    '''
    def getProperty(self, xpath,PropertyName):
        try:
            elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
            #elementVisible = super().WaitFor_VisibilityOf_Element_Located(xpath)
            if(elementPresense):
                #if(elementVisible):
                    return elementPresense.get_property(PropertyName)
            else:
                self.__raise_element_not_present_exception(xpath)                    
        except Exception as error:
            raise error             
    '''
        Whether the element is visible to a user.
    '''
    def isElementDisplayed(self, xpath):
        try:
            return self.driver.find_element_by_xpath(xpath).is_displayed()
                                
        except Exception as error:
            raise error             
    '''
        Returns whether the element is enabled.
    '''
    def isElementEnabled(self, xpath):
        try:

            return return self.driver.find_element_by_xpath(xpath).is_enabled()
                    
        except Exception as error:
            raise error
    '''
        Returns whether the element is selected.

        Can be used to check if a checkbox or radio button is selected.
    '''
    def isElementSelected(self, xpath):
        try:

                    return return self.driver.find_element_by_xpath(xpath).is_selected()
                     
        except Exception as error:
            raise error
    '''
        Saves a screenshot of the current element to a PNG image file. Returns
        False if there is any IOError, else returns True. Use full paths in your filename.
    '''
    def currentElementScreenshot(self, xpath,screenShotSavingPath):
        try:
            elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
            elementVisible = super().WaitFor_VisibilityOf_Element_Located(xpath)
            if(elementPresense):
                if(elementVisible):
                    return elementVisible.screenshot(screenShotSavingPath)
                else:
                    self.__raise_element_not_visible_exception(xpath) 
            else:
                self.__raise_element_not_present_exception(xpath)                     
        except Exception as error:
            raise error
        

    # action class

    def actionClick(self, xpath):
        try:
            actions = ActionChains(self.driver)
            elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
            elementVisible = super().WaitFor_VisibilityOf_Element_Located(xpath)
            if elementPresense :
                if elementVisible:
                    #element = self.driver.find_element(By.XPATH, xpath)
                    actions.move_to_element(elementVisible).click(elementVisible)
                    actions.perform()
                else:
                    self.__raise_element_not_visible_exception(xpath) 
            else:
                self.__raise_element_not_present_exception(xpath)                     
        except Exception as error:
            raise  error
    def action_clear_field(self, xpath):
        try:
            actions = ActionChains(self.driver)
            elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
            elementVisible = super().WaitFor_VisibilityOf_Element_Located(xpath)
            if elementPresense :
                if elementVisible:
                    #element = self.driver.find_element(By.XPATH, xpath)
                    actions.click(elementVisible).key_down(Keys.CONTROL).send_keys('a').key_up(Keys.CONTROL).send_keys(Keys.BACK_SPACE).pause(1).perform()
                else:
                    self.__raise_element_not_visible_exception(xpath) 
            else:
                self.__raise_element_not_present_exception(xpath)                     
        except Exception as error:
            raise  error                

        '''
        Holds down the left mouse button on an element.

        Args:	
        on_element: The element to mouse down. If None, clicks on current mouse position.
        '''
    def actionClickandHold(self, xpath=None):
        try:
            actions = ActionChains(self.driver)
            if(xpath != None):
                elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
                elementVisible = super().WaitFor_VisibilityOf_Element_Located(xpath)
                if elementPresense:
                    if elementVisible:
                        actions.click_and_hold(elementVisible)
                        actions.perform()
                    else:
                        self.__raise_element_not_visible_exception(xpath) 
                else:
                    self.__raise_element_not_present_exception(xpath)                         
            else:
                actions.click_and_hold()
                actions.perform()
        except Exception as error:
            raise error        
    '''
    Moves the mouse to the element
    '''
    def moveToElement(self, xpath):
        try:
            actions = ActionChains(self.driver)
            elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
            elementVisible = super().WaitFor_VisibilityOf_Element_Located(xpath)
            if elementPresense:
                if elementVisible:
                    #element = self.driver.find_element(By.XPATH, xpath)
                    actions.move_to_element(elementVisible).perform()
                else:
                    self.__raise_element_not_visible_exception(xpath) 
            else:
                self.__raise_element_not_present_exception(xpath)                     
        except Exception as error:
            raise error
                
    '''
    Moves the mouse to the element and clicks it
    '''
    def moveToElement_and_click(self, xpath):
        try:
            actions = ActionChains(self.driver)
            elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
            elementVisible = super().WaitFor_VisibilityOf_Element_Located(xpath)
            if elementPresense:
                if elementVisible:
                    #element = self.driver.find_element(By.XPATH, xpath)
                    actions.move_to_element(elementVisible).click(elementVisible).perform()
                else:
                    self.__raise_element_not_visible_exception(xpath) 
            else:
                self.__raise_element_not_present_exception(xpath)                     
        except Exception as error:
            raise error
        '''
        if both xpathOfMovingTO and xpathOfElemtToClick are provided, it will click the subemenet like below example
        ex:
        actions = ActionChains(driver)
        actions.move_to_element(menu)
        actions.click(hidden_submenu)
        actions.perform()
        if user didnt provide subelement xpath, then it clicks the same element mouse moved to
        '''    
    def moveToElement_and_subElement_click(self, xpathOfMovingTO, xpathOfElemtToClick=None):
        try:
            if(xpathOfElemtToClick!=None):    
                actions = ActionChains(self.driver)
                elementPresense_OfMovingTo = super().WaitFor_PresenseOf_Element_Located(xpathOfMovingTO)
                elementVisible_OfMovingTo = super().WaitFor_VisibilityOf_Element_Located(xpathOfMovingTO)
                elementPresense_ToClick = super().WaitFor_PresenseOf_Element_Located(xpathOfElemtToClick)
                elementVisible_ToClick = super().WaitFor_VisibilityOf_Element_Located(xpathOfElemtToClick)
                if elementPresense_OfMovingTo:
                    if elementVisible_OfMovingTo:
                        if elementPresense_ToClick:
                            if elementVisible_ToClick:
                                #MovedToelement = self.driver.find_element(By.XPATH, xpathOfMovingTO)
                                #ClickElement=  self.driver.find_element(By.XPATH, xpathOfElemtToClick)
                                actions.move_to_element(elementVisible_OfMovingTo).click(elementVisible_ToClick).perform()
                            else:
                                self.__raise_element_not_visible_exception(xpathOfElemtToClick) 
                        else:
                            self.__raise_element_not_present_exception(xpathOfElemtToClick)  
                    else:
                        self.__raise_element_not_visible_exception(xpathOfMovingTO)
                else:
                    self.__raise_element_not_present_exception(xpathOfMovingTO)

            else:
                elementPresense_OfMovingTo = super().WaitFor_PresenseOf_Element_Located(xpathOfMovingTO)
                elementVisible_OfMovingTo = super().WaitFor_VisibilityOf_Element_Located(xpathOfMovingTO)
                if elementPresense_OfMovingTo :
                    if elementVisible_OfMovingTo:
                        #MovedToelement = self.driver.find_element(By.XPATH, xpathOfMovingTO)
                      
                        actions.move_to_element(elementVisible_OfMovingTo).click().perform()
                    else:
                        self.__raise_element_not_visible_exception(xpathOfMovingTO)
                else:
                    self.__raise_element_not_present_exception(xpathOfMovingTO)                        
        except Exception as error:
            raise error 
        '''
            Move the mouse by an offset of the specified element.
            Offsets are relative to the top-left corner of the element.
            Args:	
            to_element: The WebElement to move to.
            xoffset: X offset to move to.
            yoffset: Y offset to move to.
        '''           
    def moveToElementWithOffset(self, xpath, xoffset, yoffset):
        try:
            actions = ActionChains(self.driver)
            elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
            elementVisible = super().WaitFor_VisibilityOf_Element_Located(xpath)
            if elementPresense:
                if elementVisible:
                    #element = self.driver.find_element(By.XPATH, xpath)
                    actions.move_to_element_with_offset(elementVisible, xoffset, yoffset)
                    actions.perform()
                else:
                    self.__raise_element_not_visible_exception(xpath)
            else:
                self.__raise_element_not_present_exception(xpath)                    
        except Exception as error:
            raise error
    def moveToElement_WithOffset_and_click(self, xpath, xoffset, yoffset):
        try:
            actions = ActionChains(self.driver)
            elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
            elementVisible = super().WaitFor_VisibilityOf_Element_Located(xpath)
            if elementPresense:
                if elementVisible:
                    #element = self.driver.find_element(By.XPATH, xpath)
                    actions.move_to_element_with_offset(elementVisible, xoffset, yoffset).click()
                    actions.perform()
                else:
                    self.__raise_element_not_visible_exception(xpath)
            else:
                self.__raise_element_not_present_exception(xpath)                     
        except Exception as error:
            raise error

                      
    def moveTo_location_and_click(self, xoffset, yoffset,clickElementXpath=None):
        try:
            actions = ActionChains(self.driver)

            if clickElementXpath!=None:
                elementPresense = super().WaitFor_PresenseOf_Element_Located(clickElementXpath)
                elementVisible = super().WaitFor_VisibilityOf_Element_Located(clickElementXpath)
                if elementPresense:
                    if elementVisible:
                        #element = self.driver.find_element(By.XPATH, clickElementXpath)
                        actions.move_to_location( xoffset, yoffset).click(elementVisible)
                        actions.perform()
                    else:
                        self.__raise_element_not_visible_exception(clickElementXpath)
                else:
                    self.__raise_element_not_present_exception(clickElementXpath)                         
            else:
                actions.move_to_location( xoffset, yoffset).click()
                actions.perform()                       
        except Exception as error:
            raise error  
    '''
    move_by_offset(xoffset, yoffset)¶
    Moving the mouse to an offset from current mouse position.

    Args:	
    xoffset: X offset to move to, as a positive or negative integer.
    yoffset: Y offset to move to, as a positive or negative integer.

    '''                                  
    def moveByOffsett(self, xoffset, yoffset):
        try:
            actions = ActionChains(self.driver)
            actions.move_by_offset(xoffset, yoffset)
            actions.perform()
        except Exception as error:
            raise error
    def move_ByOffset_and_click(self, xoffset, yoffset):
        try:
            actions = ActionChains(self.driver)
            actions.move_by_offset(xoffset, yoffset).click()
            actions.perform()
        except Exception as error:
            raise error                
    '''         
    def actionRelease(self, xpath=None):
        try:
            actions = ActionChains(self.driver)
            if(xpath != None):
                if(super().WaitFor_PresenseOf_Element_Located(xpath)):
                    element = self.driver.find_element(By.XPATH, xpath)
            else:
                element = None
            if(element != None):
                actions.release(element)
                actions.perform()

            else:
                actions.release()
                actions.perform()
        except Exception as error:
            raise error        
    '''

    def keyDown_and_sendKeys(self, ModifierKey:Keys, keys, xpath=None):
        try:
            actions = ActionChains(self.driver)
            if(xpath != None):
                elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
                elementVisible = super().WaitFor_VisibilityOf_Element_Located(xpath)
                if elementPresense:
                    if elementVisible:
                        #element = self.driver.find_element(By.XPATH, xpath)
                        actions.key_down(ModifierKey, elementVisible).send_keys(keys)
                        actions.perform()
                    else:
                        self.__raise_element_not_visible_exception(xpath)
                else:
                    self.__raise_element_not_present_exception(xpath)                        
            else:
                actions.key_down(ModifierKey).send_keys(keys)
                actions.perform()
        except Exception as error:
            raise error

    '''
    Releases a modifier key.

    Args:	
    value: The modifier key to send. Values are defined in Keys class.
    element: The element to send keys. If None, sends a key to current focused element.
    Example, pressing ctrl+c:

    ActionChains(driver).key_down(Keys.CONTROL).send_keys('c').key_up(Keys.CONTROL).perform()
    
    '''
    def keyUp(self, ModifierKey:Keys, xpath=None):
        try:
            actions = ActionChains(self.driver)
            if(xpath != None):
                elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
                elementVisible = super().WaitFor_VisibilityOf_Element_Located(xpath)
                if elementPresense:
                    if elementVisible:
                        #element = self.driver.find_element(By.XPATH, xpath)
                        actions.key_up(ModifierKey, elementVisible).perform()
                    else:
                        self.__raise_element_not_visible_exception(xpath)
                else:
                    self.__raise_element_not_present_exception(xpath)                        
            else:

                actions.key_up(ModifierKey).perform()
        except Exception as error:
            raise error  
        '''
         keyboard key press down --> keyboard send keys -->keyboard releasing the key 
        '''
    def keyDown_sendKeys_keyUP(self,ModifierKey:Keys,keys:str, xpath=None):
        try:
            actions = ActionChains(self.driver)   
            if(xpath != None):
                elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
                elementVisible = super().WaitFor_VisibilityOf_Element_Located(xpath)
                if elementPresense:
                    if elementVisible:
                        #element = self.driver.find_element(By.XPATH, xpath)
                        actions.key_down(ModifierKey,elementVisible).send_keys(keys).key_up(ModifierKey,elementVisible).perform()
                    else:
                        self.__raise_element_not_visible_exception(xpath)
                else:
                    self.__raise_element_not_present_exception(xpath)                        
            else:
                actions.key_down(ModifierKey).send_keys(keys).key_up(ModifierKey).perform()
        except Exception as error:
            raise error    



    '''
    Double-clicks an element.

    Args:	
    on_element: The element to double-click. If None, clicks on current mouse position.
    '''
    def double_click(self, xpath: None):
        try:
            actions = ActionChains(self.driver)
            if(xpath != None):
                elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
                elementVisible = super().WaitFor_VisibilityOf_Element_Located(xpath)
                if elementPresense:
                    if elementVisible:
                        #element = self.driver.find_element(By.XPATH, xpath)
                        actions.double_click(elementVisible)
                        actions.perform()
                    else:
                        self.__raise_element_not_visible_exception(xpath)
                else:
                    self.__raise_element_not_present_exception(xpath)
            else:
                actions.double_click()
                actions.perform()
        except Exception as error:
            raise error        

        '''
        Performs a context-click (right click) on an element.

        Args:	
        on_element: The element to context-click. If None, clicks on current mouse position.
        '''
    def right_Click(self, xpath=None):
        try:
            actions = ActionChains(self.driver)
            if(xpath != None):
                elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
                elementVisible = super().WaitFor_VisibilityOf_Element_Located(xpath)
                if elementPresense:
                    if elementVisible:
                        actions.context_click(elementVisible)
                        actions.perform()
                    else:
                        self.__raise_element_not_visible_exception(xpath)
                else:
                    self.__raise_element_not_present_exception(xpath)
            else:
                actions.context_click()
                actions.perform() 
        except Exception as error:
            raise error               

    def resetActions(self):
        try:
            actions = ActionChains(self.driver)
            actions.reset_actions()
            actions.perform()
        except Exception as error:
            raise error            

    def sendKeys(self, *keys_to_send):
        try:
            actions = ActionChains(self.driver)
            actions.send_keys(*keys_to_send)
            actions.perform()
        except Exception as error:
            raise error            

    def sendKeysToElement(self, xpath, *keys_to_send):
        try:
            actions = ActionChains(self.driver)
            elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
            elementVisible = super().WaitFor_VisibilityOf_Element_Located(xpath)
            if elementPresense:
                if elementVisible:
                    #element = super().WaitFor_PresenseOf_Element_Located(xpath)
                    actions.click(elementVisible)
                    actions.send_keys_to_element(elementVisible, *keys_to_send)
                    actions.perform()
                else:
                    self.__raise_element_not_visible_exception(xpath)
            else:
                self.__raise_element_not_present_exception(xpath)                    
        except Exception as error:
            raise error    

        '''
        Sends wheel scroll information to the driver to be processed.

        Args:	
        x: starting X coordinate
        y: starting Y coordinate
        delta_x: the distance the mouse will scroll on the x axis
        delta_y: the distance the mouse will scroll on the y axis

        '''
    def Scroll(self,x: int, y: int, delta_x: int, delta_y: int, duration: int = 0, origin: str = 'viewport'):
        try:
            actions = ActionChains(self.driver)
            
            actions.scroll(x, y, delta_x, delta_y, duration, origin)
            actions.perform()
        except Exception as error:
            raise error 

        '''
        Scrolls by provided amounts with the origin in the top left corner of the viewport.

    Args:	
    delta_x: Distance along X axis to scroll using the wheel. A negative value scrolls left.
    delta_y: Distance along Y axis to scroll using the wheel. A negative value scrolls up.
        '''
    def Scroll_by_amount(self,delta_x: int, delta_y: int):
        try:
            actions = ActionChains(self.driver)
            
            actions.scroll_by_amount( delta_x, delta_y)
            actions.perform()
        except Exception as error:
            raise error
        '''
        Scrolls by provided amount based on a provided origin. The scroll origin is either the center of an 
        element or the upper left of the viewport plus any offsets. If the origin is an element, 
        and the element is not in the viewport, the bottom of the element will first be scrolled to the bottom of the viewport.

        Args:	
        origin: Where scroll originates (viewport or element center) plus provided offsets.
        delta_x: Distance along X axis to scroll using the wheel. A negative value scrolls left.
        delta_y: Distance along Y axis to scroll using the wheel. A negative value scrolls up.
        Raises:	If the origin with offset is outside the viewport. - MoveTargetOutOfBoundsException - If the origin with offset 
        is outside the viewport.
        '''
    def Scroll_from_origin(self,scroll_origin: ScrollOrigin,delta_x: int, delta_y: int):
        try:
            actions = ActionChains(self.driver)
            
            actions.scroll_from_origin( scroll_origin,delta_x, delta_y)
            actions.perform()
        except Exception as error:
            raise error


        '''
                If the element is outside the viewport, scrolls the bottom of the element to the bottom of the viewport.

        Args:	
        element: Which element to scroll into the viewport.
        send_keys(*keys_to_send)
        Sends keys to current focused element.

        Args:	
        keys_to_send: The keys to send. Modifier keys constants can be found in the ‘Keys’ class.
        '''
    def scroll_to_Element(self, xpath):
        try:
            actions = ActionChains(self.driver)
            elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
            #elementVisible = super().WaitFor_VisibilityOf_Element_Located(xpath)
            if elementPresense:
                #if elementVisible:
                    #element = self.driver.find_element(By.XPATH, xpath)
                    actions.scroll_to_element( elementPresense)
                    actions.perform()
            else:
                self.__raise_element_not_present_exception(xpath)        
        except Exception as error:
            raise error
    def switch_To_ActiveElement(self):
        try:

            self.driver.switch_to.active_element
        except Exception as error:
            raise error
    def formSubmit(self,xpath):
        try:
            elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
            elementVisible = super().WaitFor_VisibilityOf_Element_Located(xpath)
            if elementPresense:
                if elementVisible:
                    elementVisible.submit()
                else:
                    self.__raise_element_not_visible_exception(xpath)
            else:
                self.__raise_element_not_present_exception(xpath)                     
        except Exception as error:
            raise error
    def get_Value_of_css_property(self,xpath,property_name):
        try:
            elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
            #elementVisible = super().WaitFor_VisibilityOf_Element_Located(xpath)
            if elementPresense:
                #if elementVisible:
                    return elementPresense.value_of_css_property(property_name)
            else:
                self.__raise_element_not_present_exception(xpath)                     
        except Exception as error:
            raise error
    def get_Accessible_name(self,xpath):
        try:
            elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
            #elementVisible = super().WaitFor_VisibilityOf_Element_Located(xpath)
            if elementPresense:
                #if elementVisible:
                    return elementPresense.accessible_name
            else:
                self.__raise_element_not_present_exception(xpath)                     
        except Exception as error:
            raise error 
    def get_Aria_role(self,xpath):
        try:
            elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
            #elementVisible = super().WaitFor_VisibilityOf_Element_Located(xpath)
            if elementPresense:
                #if elementVisible:
                    return elementPresense.Aria_role
            else:
                self.__raise_element_not_present_exception(xpath)                     
        except Exception as error:
            raise error 
    '''Internal ID used by selenium.

    Internal ID used by selenium.

    This is mainly for internal use. Simple use cases such as checking if 2 webelements refer to the same element, can be done using ==:

    if element1 == element2:
        print("These 2 are equal")
    '''                                                  
    def Get_internal_ID(self,xpath):
        try:
            elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
            #elementVisible = super().WaitFor_VisibilityOf_Element_Located(xpath)
            if elementPresense:
                #if elementVisible:
                    return elementPresense.id
            else:
                self.__raise_element_not_present_exception(xpath)                     
        except Exception as error:
            raise error
    def Get_location_of_Element(self,xpath):
        try:
            elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
            #elementVisible = super().WaitFor_VisibilityOf_Element_Located(xpath)
            if elementPresense:
                #if elementVisible:
                    return elementPresense.location
            else:
                self.__raise_element_not_present_exception(xpath)                     
        except Exception as error:
            raise error
    '''
    THIS PROPERTY MAY CHANGE WITHOUT WARNING. Use this to discover where on the screen an element is 
    so that we can click it. This method should cause the element to be scrolled into view.

    Returns the top lefthand corner location on the screen, or None if the element is not visible.
    '''
    def get_Scroll_location_of_Element(self,xpath):
        try:
            elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
            #elementVisible = super().WaitFor_VisibilityOf_Element_Located(xpath)
            if elementPresense:
                #if elementVisible:
                    return elementPresense.location_once_scrolled_into_view
            else:
                self.__raise_element_not_present_exception(xpath)                     
        except Exception as error:
            raise error
    def scroll_into_view_by_js(self,xpath):
        elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath) 
        if elementPresense:       
            self.driver.execute_script("arguments[0].scrollIntoView(true);", self.driver.find_element_by_xpath(xpath))  
        else:
            self.__raise_element_not_present_exception(xpath) 
    def clear_input_field_by_js(self,xpath):
        elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath) 
        if elementPresense:       
            self.driver.execute_script("arguments[0].value = '';", self.driver.find_element_by_xpath(xpath))  
        else:
            self.__raise_element_not_present_exception(xpath)
            
    def click_by_js(self,xpath):
        elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath) 
        if elementPresense:       
            self.driver.execute_script("arguments[0].click();", self.driver.find_element_by_xpath(xpath))  
        else:
            self.__raise_element_not_present_exception(xpath)                 

    '''
    Internal reference to the WebDriver instance this element was found from.
    '''        
    def get_Prent_of_Element(self,xpath):
        try:
            elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
            #elementVisible = super().WaitFor_VisibilityOf_Element_Located(xpath)
            if elementPresense:
                #if elementVisible:
                    return elementPresense.parent
            else:
                self.__raise_element_not_present_exception(xpath)                 
        except Exception as error:
            raise error
    '''
    A dictionary with t he size and location of the element.
    '''        
    def get_size_And_Location(self,xpath):
        try:
            elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
            #elementVisible = super().WaitFor_VisibilityOf_Element_Located(xpath)
            if elementPresense:
                #if elementVisible:
                    return elementPresense.rect
            else:
                self.__raise_element_not_present_exception(xpath)                     
        except Exception as error:
            raise error
    def get_element_screenshot_as_base64(self,xpath,name:str):
        try:
            elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
            elementVisible = super().WaitFor_VisibilityOf_Element_Located(xpath)
            if elementPresense:
                if elementVisible:
                    with open(self.__webElement_Screenshot_Location()+"/"+name+".jpg", "wb") as fh:
                        fh.write(base64.urlsafe_b64decode(elementVisible.screenshot_as_base64))
                else:
                    self.__raise_element_not_visible_exception(xpath)
            else:
                self.__raise_element_not_present_exception(xpath)            
            #return element.screenshot_as_base64
        except Exception as error:
            raise error
    def get_element_screenshot_as_png(self,xpath,name:str):
        try:
            elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
            elementVisible = super().WaitFor_VisibilityOf_Element_Located(xpath)
            if elementPresense:
                if elementVisible:
                    result_File=self.__webElement_Screenshot_Location()+"/"+name+".png"
                    with open(result_File, "wb") as fh:
                        fh.write(elementVisible.screenshot_as_png)
                        #Image.open(result_File).save(result_File, 'PNG')
                else:
                    self.__raise_element_not_visible_exception(xpath)
            else:
                self.__raise_element_not_present_exception(xpath)                        
        except Exception as error:
            raise error
    '''
    shadow_root¶
    Returns a shadow root of the element if there is one or an error. Only works from Chromium 96 onwards. Previous versions of Chromium based browsers will throw an assertion exception.

    Returns:	
    ShadowRoot object or
    NoSuchShadowRoot - if no shadow root was attached to element
    
    '''        
    def get_Element_shadow_root(self,xpath):
        try:
            elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
            #elementVisible = super().WaitFor_VisibilityOf_Element_Located(xpath)
            if elementPresense:
               # if elementVisible:
                    return elementPresense.shadow_root
            else:
                self.__raise_element_not_present_exception(xpath)                    
        except Exception as error:
            raise error
    def get_Element_size(self,xpath):
        try:
            elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
            #elementVisible = super().WaitFor_VisibilityOf_Element_Located(xpath)
            if elementPresense:
                #if elementVisible:
                        return elementPresense.size
            else:
                self.__raise_element_not_present_exception(xpath)                        
        except Exception as error:
            raise error
    def get_Element_tag_name(self,xpath):
        try:
            elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
            #elementVisible = super().WaitFor_VisibilityOf_Element_Located(xpath)
            if elementPresense:
                #if elementVisible:
                    return elementPresense.tag_name
            else:
                self.__raise_element_not_present_exception(xpath)                    
        except Exception as error:
            raise error
    def click_by_js(self,xpath):
        try:
            elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
            self.driver.execute_script("arguments[0].click();", self.driver.find_element_by_xpath(xpath))
        except Exception as error:
            raise error

    def get_Element_text(self,xpath):
        try:
            elementPresense = super().WaitFor_PresenseOf_Element_Located(xpath)
            #elementVisible = super().WaitFor_VisibilityOf_Element_Located(xpath)
            if elementPresense:
                #if elementVisible:
                    return elementPresense.text
            else:
                self.__raise_element_not_present_exception(xpath)                    
        except Exception as error:
            raise error
    def __webElement_Screenshot_Location(self):
        try:
            dir_name=os.getcwd()+"/webElement_Screenshots"
            os.makedirs(dir_name, exist_ok=True)
            return dir_name
        except FileExistsError:
            pass
    def __with_findElement_withAnyLocator(self, locatorType, locator):

        #if locatorType == "XPATH":
        try:
            element = super().WaitFor_PresenseOf_Element_Located_AnyLocatorType(locatorType,locator)
                
            return element
        except Exception as error:
            return error
         
    def __raise_element_not_present_exception(self,locator):
        raise Exception("element not present : "+locator)

    def __raise_element_not_visible_exception(self,locator):
        raise Exception("element not visible : "+locator)             
        


from select import select
from xml.dom.minidom import Element
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select
from selenium.webdriver import ActionChains
from selenium.webdriver.common.alert import Alert
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support.select import Select
from pyallied.web.webWaits import customwebDriverwait
# drop downs


class DropDownActions(customwebDriverwait):
    def __init__(self, driver):
        super().__init__(driver)
        self.driver = driver

    def selectDropDownByValue(self, xpath, valueToSelect):
        try:
            elementPresense=super().WaitFor_PresenseOf_Element_Located(xpath)
            #elementVisibility=super().WaitFor_VisibilityOf_Element_Located(xpath)
            if(elementPresense):
                #if(elementVisibility):
                    select = Select(self.driver.find_element(By.XPATH, xpath))
                    select.select_by_value(valueToSelect)
            else:
                self.__raise_element_not_present_exception(xpath)                    
        except Exception as error:
            raise error
    def selectDropDownByIndex(self, xpath, indexToSelect):
        try:
            elementPresense=super().WaitFor_PresenseOf_Element_Located(xpath)
            #elementVisibility=super().WaitFor_VisibilityOf_Element_Located(xpath)
            if(elementPresense):
                #if(elementVisibility):
                    select = Select(self.driver.find_element(By.XPATH, xpath))
                    select.select_by_index(indexToSelect)
            else:
                self.__raise_element_not_present_exception(xpath)                    
        except Exception as error:
            raise error
    def selectDropDownByVisibleText(self, xpath, textToSelect):
        try:
            elementPresense=super().WaitFor_PresenseOf_Element_Located(xpath)
            #elementVisibility=super().WaitFor_VisibilityOf_Element_Located(xpath)
            if(elementPresense):
                    select = Select(self.driver.find_element(By.XPATH, xpath))
                    select.select_by_visible_text(textToSelect)
            else:
                self.__raise_element_not_present_exception(xpath)                    
        except Exception as error:
            raise error
    def deselectAllOptionsInDropDown(self, xpath):
        try:
            elementPresense=super().WaitFor_PresenseOf_Element_Located(xpath)
            #elementVisibility=super().WaitFor_VisibilityOf_Element_Located(xpath)
            if(elementPresense):
                #if(elementVisibility):
                    select = Select(self.driver.find_element(By.XPATH, xpath))
                    select.deselect_all()
            else:
                self.__raise_element_not_present_exception(xpath)                    
        except Exception as error:
            raise error
    def getDefaultSelectedDropDownOptions(self, xpath):
        try:
            elementPresense=super().WaitFor_PresenseOf_Element_Located(xpath)
            #elementVisibility=super().WaitFor_VisibilityOf_Element_Located(xpath)
            selectedOptions=[]
            if(elementPresense):
                #if(elementVisibility):
                    select = Select(self.driver.find_element(By.XPATH, xpath))
                    all_selected_options = select.all_selected_options
                    for i in all_selected_options:
                        if(i.text):
                            selectedOptions.append(i.text)
                        elif(i.get_attribute('value')):
                            selectedOptions.append(i.get_attribute('value'))
                    return selectedOptions
            else:
                self.__raise_element_not_present_exception(xpath)                    
        except Exception as error:
            raise error
    def getAllOptionInDropDown(self, xpath):
        try:
            elementPresense=super().WaitFor_PresenseOf_Element_Located(xpath)
            #elementVisibility=super().WaitFor_VisibilityOf_Element_Located(xpath)
            allOptions=[]
            if(elementPresense):
                #if(elementVisibility):
                    select = Select(self.driver.find_element(By.XPATH, xpath))
                    options = select.options
                    for i in options:
                        if(i.text):
                            allOptions.append(i.text)
                        elif(i.get_attribute('value')):
                            allOptions.append(i.get_attribute('value'))

                    return allOptions
            else:
                self.__raise_element_not_present_exception(xpath)                    
        except Exception as error:
            raise error
    def deselectByIndex(self, xpath, index):
        try:
            elementPresense=super().WaitFor_PresenseOf_Element_Located(xpath)
            #elementVisibility=super().WaitFor_VisibilityOf_Element_Located(xpath)
            if(elementPresense):
                #if(elementVisibility):
                    select = Select(self.driver.find_element(By.XPATH, xpath))
                    select.deselect_by_index(index)
            else:
                self.__raise_element_not_present_exception(xpath)                    
        except Exception as error:
            raise error
    def deselectByValue(self, xpath, value):
        try:
            elementPresense=super().WaitFor_PresenseOf_Element_Located(xpath)
            #elementVisibility=super().WaitFor_VisibilityOf_Element_Located(xpath)
            if(elementPresense):
                #if(elementVisibility):
                    select = Select(self.driver.find_element(By.XPATH, xpath))
                    select.deselect_by_value(value)
            else:
                self.__raise_element_not_present_exception(xpath)                    
        except Exception as error:
            raise error
    def deselectByVisibleText(self, xpath, text):
        try:
            elementPresense=super().WaitFor_PresenseOf_Element_Located(xpath)
            #elementVisibility=super().WaitFor_VisibilityOf_Element_Located(xpath)
            if(elementPresense):
                #if(elementVisibility):        
                    select = Select(self.driver.find_element(By.XPATH, xpath))
                    select.deselect_by_visible_text(text)
            else:
                self.__raise_element_not_present_exception(xpath)                    
        except Exception as error:
            raise error
    def getFirstSelecteOption(self, xpath):
        try:
            elementPresense=super().WaitFor_PresenseOf_Element_Located(xpath)
            #elementVisibility=super().WaitFor_VisibilityOf_Element_Located(xpath)
            firstSelectedOption=[]
            if(elementPresense):
                #if(elementVisibility):        
                    select = Select(self.driver.find_element(By.XPATH, xpath))
                    selectedOption=select.first_selected_option
                    #for i in selectedOption:
                    if(selectedOption.text):
                            firstSelectedOption.append(selectedOption.text)
                    elif(selectedOption.get_attribute('value')):
                            firstSelectedOption.append(selectedOption.get_attribute('value'))

                    return firstSelectedOption 
            else:
                self.__raise_element_not_present_exception(xpath)                    
        except Exception as error:
            raise error                    
    def __raise_element_not_present_exception(self,locator):
        raise Exception("element not present : "+locator)

    def __raise_element_not_visible_exception(self,locator):
        raise Exception("element not visible : "+locator)  


import ast
import requests
import xlrd
import pandas as pd
import json
import numpy as np
import base64
import tweepy

class crud:

    def __init__(self):
        pass
    #def twitterPost(self):


    def crudCall(self,testLocation,testcaseId):
        #"D:/Users/sjyothi/Repos/SeleniumActionBot/pyseleniumbot/api/apiList.xlsx","Test_1"
        print(testcaseId)
        dictdata=crud.__readData_toDict(self,testLocation,testcaseId)
        headersDict={}
        #print(dictdata)
        dataH=str(dictdata["headers"]).split(",")
        for data in dataH:
            headersDict[str(data.split(":")[0])]=str(data.split(":")[1])
            #print(str(data.split(":")[0]))

        #print(headersDict)
        #print(ast.literal_eval(str(dictdata["headers"]).replace("‘","'")))
        if dictdata["crud_type"]=="POST":
            if( "multipart" in dictdata["headers"]):
                files = {'file': open(dictdata["multipart_data"], 'rb')}
                f=open(dictdata["data"])
                

                
                data=json.dumps(json.load(f)[testcaseId])
                
                respo=requests.post(dictdata["api_signature"],headers=headersDict,data=data,files=files)
                #print(respo.json())
                return respo
                print('')
            else:
                f=open(dictdata["data"])
                
                data=json.dumps(json.load(f)[testcaseId])
                
                respo=requests.post(dictdata["api_signature"],headers=headersDict,data=data)
                #print(respo.json())
                return respo
        if dictdata["crud_type"]=="GET":
            #f=open(dictdata["data"])
            
            #data=json.dumps(json.load(f)[testcaseId])
            
            respo=requests.get(dictdata["api_signature"],headers=headersDict)
            
            #print(respo.json())
            return respo
        if dictdata["crud_type"]=="DELETE":
            #f=open(dictdata["data"])
            
            #data=json.dumps(json.load(f)[testcaseId])
            
            respo=requests.delete(dictdata["api_signature"])
            
            #print(respo.json())
            return respo
        if dictdata["crud_type"]=="PUT":
            f=open(dictdata["data"])
                
            data=json.dumps(json.load(f)[testcaseId])
                
            respo=requests.put(dictdata["api_signature"],headers=headersDict,data=data)
            return respo                        





    def __readData(self,dataLocation:str):
        # Give the location of the file
        loc = ("path of file")
        
        # To open Workbook
        wb = xlrd.open_workbook(dataLocation)
        sheet = wb.sheet_by_index(0)
    
        
        # For row 0 and column 0
        #print(sheet.row_values(1))
        return sheet.cell_value(0, 0)
    def __readData_toDict(self,dataLocation:str,testcaseid:str):
        '''
        xls = pd.read_excel(dataLocation)
        dataDict=xls.set_index("testcase_id").to_dict("index")
        dataJson= xls.set_index("testcase_id").to_json(orient="index")
        
        jsonFormat=json.loads(dataJson)

        return dataDict.get(testcaseid)
        '''
        xls = pd.ExcelFile(dataLocation)
        
        sheets = xls.sheet_names
        for sheet in sheets:
            xls = pd.read_excel(dataLocation,sheet_name=sheet)
            dataDict=xls.set_index("testcase_id").to_dict("index")
            if(dataDict.get(testcaseid)):
                print(dataDict.get(testcaseid))
                return dataDict.get(testcaseid)
                        
        xls.close()


import polling2, time
from appium.webdriver.common.appiumby import AppiumBy
from selenium.common.exceptions import NoSuchElementException

class pollWait:
    def __init__(self,driver):
          self.driver=driver
    def with_findElements_byXpath(self,xpath):
        try:
            polling2.poll(lambda: self.driver.find_elements(AppiumBy.XPATH, xpath),ignore_exceptions=(NoSuchElementException,), step=5, timeout=30)
        except Exception:
            return Exception
    def with_findElement_byXpath(self,xpath):
        try:
            polling2.poll(lambda: self.driver.find_element(AppiumBy.XPATH, xpath),ignore_exceptions=(NoSuchElementException,), step=5, timeout=30)
        except Exception:
            return Exception
           


import setuptools

with open("README.md", "rb") as fh:
    long_description = fh.read().decode("utf8").replace('\r\n', '\n')

setuptools.setup(
    name="pyallied",                     # This is the name of the package
    version="1.0.15",                        # The initial release version
    author="srinivasaraojyothi@gmail.com",                     # Full name of the author
    description="pyallied - a wrapper based on selenium python",
    url = "https://github.com/srinivasaraojyothi/seleniumpythonactionbot.git",
    long_description=long_description,      # Long description read from the the readme file
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),    # List of all python modules to be installed
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],                                      # Information to filter the project on PyPi website
    python_requires='>=3.9',                # Minimum version requirement of the package
    py_modules=["dragAndDrop","alerts","DropDown","webWaits","windowAndFrame","webElement","pollWait","session","mobAppium"],             # Name of the python package
    package_dir={'pyallied.web':'pyallied/web', 'pyallied.mobile':'pyallied/mobile'},     # Directory of the source code of the package
    install_requires=[]                     # Install other dependencies if any
)


#!/usr/bin/env python

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import TYPE_CHECKING

from appium.common.helper import extract_const_attributes
from appium.common.logger import logger
from appium.protocols.webdriver.can_execute_commands import CanExecuteCommands
from appium.webdriver.mobilecommand import MobileCommand as Command

if TYPE_CHECKING:
    from appium.webdriver.webdriver import WebDriver


class GsmCallActions:
    CALL = 'call'
    ACCEPT = 'accept'
    CANCEL = 'cancel'
    HOLD = 'hold'


class GsmSignalStrength:
    NONE_OR_UNKNOWN = 0
    POOR = 1
    MODERATE = 2
    GOOD = 3
    GREAT = 4


class GsmVoiceState:
    UNREGISTERED = 'unregistered'
    HOME = 'home'
    ROAMING = 'roaming'
    SEARCHING = 'searching'
    DENIED = 'denied'
    OFF = 'off'
    ON = 'on'


class Gsm(CanExecuteCommands):
    def make_gsm_call(self, phone_number: str, action: str) -> 'WebDriver':
        """Make GSM call (Emulator only)

        Android only.

        Args:
            phone_number: The phone number to call to.
            action: The call action.
                A member of the const `appium.webdriver.extensions.android.gsm.GsmCallActions`

        Usage:
            self.driver.make_gsm_call('5551234567', GsmCallActions.CALL)

        Returns:
            Union['WebDriver', 'Gsm']: Self instance
        """
        constants = extract_const_attributes(GsmCallActions)
        if action not in constants.values():
            logger.warning(
                f'{action} is unknown. Consider using one of {list(constants.keys())} constants. '
                f'(e.g. {GsmCallActions.__name__}.CALL)'
            )
        self.execute(Command.MAKE_GSM_CALL, {'phoneNumber': phone_number, 'action': action})
        return self  # type: ignore

    def set_gsm_signal(self, strength: int) -> 'WebDriver':
        """Set GSM signal strength (Emulator only)

        Android only.

        Args:
            strength: Signal strength.
                A member of the enum :obj:`appium.webdriver.extensions.android.gsm.GsmSignalStrength`

        Usage:
            self.driver.set_gsm_signal(GsmSignalStrength.GOOD)

        Returns:
            Union['WebDriver', 'Gsm']: Self instance
        """
        constants = extract_const_attributes(GsmSignalStrength)
        if strength not in constants.values():
            logger.warning(
                f'{strength} is out of range. Consider using one of {list(constants.keys())} constants. '
                f'(e.g. {GsmSignalStrength.__name__}.GOOD)'
            )
        self.execute(Command.SET_GSM_SIGNAL, {'signalStrength': strength, 'signalStrengh': strength})
        return self  # type: ignore

    def set_gsm_voice(self, state: str) -> 'WebDriver':
        """Set GSM voice state (Emulator only)

        Android only.

        Args:
            state: State of GSM voice.
                A member of the const `appium.webdriver.extensions.android.gsm.GsmVoiceState`

        Usage:
            self.driver.set_gsm_voice(GsmVoiceState.HOME)

        Returns:
            Union['WebDriver', 'Gsm']: Self instance
        """
        constants = extract_const_attributes(GsmVoiceState)
        if state not in constants.values():
            logger.warning(
                f'{state} is unknown. Consider using one of {list(constants.keys())} constants. '
                f'(e.g. {GsmVoiceState.__name__}.HOME)'
            )
        self.execute(Command.SET_GSM_VOICE, {'state': state})
        return self  # type: ignore

    def _add_commands(self) -> None:
        # noinspection PyProtectedMember,PyUnresolvedReferences
        commands = self.command_executor._commands
        commands[Command.MAKE_GSM_CALL] = ('POST', '/session/$sessionId/appium/device/gsm_call')
        commands[Command.SET_GSM_SIGNAL] = (
            'POST',
            '/session/$sessionId/appium/device/gsm_signal',
        )
        commands[Command.SET_GSM_VOICE] = ('POST', '/session/$sessionId/appium/device/gsm_voice')
